#include "cppdefs.h"
#if defined PSOURCE && !defined ANA_PSOURCE

      subroutine get_psrcs (ierr)

! Read in point sources of flow and tracer at the appropriate
! time from psource NetCDF files.  Written by Eileen Deng-Idica
! (05-03-2007) & modified by Yusuke Uchiyama (06-20-2011)

      implicit none
# include "param.h"
# include "forces.h"
# include "sources.h"
# include "scalars.h"
# include "netcdf.inc"
# include "ncvars.h"
# ifdef MPI
#  include "hidden_mpi_vars.h"
# endif
c**
c      character*64 frcname
c      integer QbarG_id
c      integer ncidfpsrc, ntpsrc,  psrc_tid, itpsrc, psrc_ncycle,
c     &                             psrc_rec, TsrcG_id(NT)
c      real psrc_time(2), psrc_cycle
c      real TsrcG(Msrc,2,NT)

    
c**
      real cff, csum, cff1,cff2
      integer ierr, lstr,lvar,lenstr,ifrc,Npas
      integer is, itrc, ncid, varid, i,j,k, it1,it2
#ifdef MPI
     &                        , i1,i2,j1,j2
#endif
      integer, parameter :: pdims=2
      integer            :: start(pdims), count(pdims)

      real,    dimension(:,:), allocatable :: D
      integer, dimension(:,:), allocatable :: C
      character(len=7)  snsrc, sk
      character(len=64) fmt
      character*(max_name_size) frcname

c# define CR

! Initialization: Inquire about the contents of psource NetCDF file:
!================  variables and dimensions. Check for consistency.

      if (may_day_flag.ne.0) return      !-->  EXIT

      lstr=lenstr(frcname)
      ncid=ncidfpsrc
      ierr=nf_noerr

      if (iic.eq.ntstart) then

# if defined BULK_FLUX
      ifrc=max(ncidfpsrc,1)
 100  ncid=ncidbulk(ifrc)
      ierr=nf_noerr
      frcname=blkfile(ifrc)
# else
      ifrc=max(ncidfpsrc,1)
 100  ncid=ncidfrc(ifrc)
      ierr=nf_noerr
      frcname=frcfile(ifrc)
# endif

!      if (iic.eq.ntstart) then
        lstr=lenstr(frcname)
        if (ncid.eq.-1) then
# if defined BULK_FLUX
          ierr=nf_open(frcname(1:lstr), nf_nowrite, ncid)
          if (ierr.eq.nf_noerr) then
            ncidbulk(ifrc)=ncid
          else
            write(*,'(/1x,4A/)') '### ERROR ',
     &                   'in get_psrcs: can not open netCDF file ''',
     &                                       frcname(1:lstr), '''.'
          endif
# else

          ierr=nf_open(frcname(1:lstr), nf_nowrite, ncid)
          if (ierr.eq.nf_noerr) then
            ncidfrc(ifrc)=ncid
          else
            write(*,'(/1x,4A/)') '### ERROR ',
     &                   'in get_psrcs: can not open netCDF file ''',
     &                                       frcname(1:lstr), '''.'
          endif
# endif

        endif
!      endif
      write(*,*) 'source_file_name = ',frcname(1:lstr)

! Nsrc: number of point sources
        ierr=nf_inq_dimid (ncid, 'Nsrc', varid)
        if (ierr .eq. nf_noerr) then
          ierr=nf_inq_dimlen (ncid, varid, Nsrc)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'Nsrc', frcname(1:lstr)
            goto 99                                      !--> ERROR
          else
            if (Nsrc.gt.Msrc) then
              write(*,*) 'FATAL ERROR: get_psrcs :: Found Nsrc > Msrc'
              goto 99                                    !--> ERROR
            endif
          endif
        else
          write(*,1) 'Nsrc', frcname(1:lstr)
          goto 99                                        !--> ERROR
        endif

        write(snsrc,'(i7)') Nsrc
        if (.not.allocated(C)) allocate (C(Nsrc,NT))
        if (.not.allocated(D)) allocate (D(Nsrc,N))




! Npas: number of tracers
        ierr=nf_inq_dimid (ncid, 'Npas', varid)
        if (ierr .eq. nf_noerr) then
          ierr=nf_inq_dimlen (ncid, varid, Npas)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'Npas', frcname(1:lstr)
            goto 99                                      !--> ERROR
          else
            if (Npas.gt.NT) then
              write(*,*) 'FATAL ERROR: get_psrcs :: Found Npas > NT'
              goto 99                                    !--> ERROR
            endif
          endif
        else
          write(*,1) 'Npas', frcname(1:lstr)
          goto 99                                        !--> ERROR
        endif

!        write(*,*) 'Nsrc = ',Nsrc ,' Msrc = ',Msrc
! Lsrc (Nsrc,itrc): logical switch

        Lsrc =.false.
        ierr=nf_inq_varid (ncid, 'Lsrc', varid)
        if (ierr .eq. nf_noerr) then
          ierr=nf_get_var_int (ncid, varid, C)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'Lsrc', frcname(1:lstr)
            goto 99                                      !--> ERROR
          endif
          do itrc=1,NT
            do is=1,Nsrc
              if (C(is,itrc).eq.1) Lsrc(is,itrc)=.true.
            enddo
          enddo
        else
          write(*,1) 'Lsrc id', frcname(1:lstr)
          goto 99                                        !--> ERROR
        endif
CR
CR        mpi_master_only write(*,*) 'get_psrcs: Lsrc'
CR        fmt = '(6x,i4,' / / snsrc / / 'i2)'
CR        do j=1,NT
CR          mpi_master_only write(*,fmt) j,(C(i,j),i=1,Nsrc)
CR        enddo
CR

! Isrc & Jsrc (Nsrc): point source locations relative to the global grid

        ierr=nf_inq_varid (ncid, 'Isrc', varid)
        if (ierr .eq. nf_noerr) then
          ierr=nf_get_var_int (ncid, varid, Isrc)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'Isrc', frcname(1:lstr)
            goto 99                                      !--> ERROR
          endif
        else
          write(*,1) 'Isrc id', frcname(1:lstr)
          goto 99                                        !--> ERROR
        endif

        ierr=nf_inq_varid (ncid, 'Jsrc', varid)
        if (ierr .eq. nf_noerr) then
          ierr=nf_get_var_int (ncid, varid, Jsrc)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'Jsrc', frcname(1:lstr)
            goto 99                                      !--> ERROR
          endif
        else
          write(*,1) 'Jsrc id', frcname(1:lstr)
          goto 99                                        !--> ERROR
        endif

# ifdef MPI

! Initially indices "Isrc,Jsrc" are assumed to be defined relative
! to the absolute physical grid. Translate them into would be local
! indices within the current MPI-subdomain and check whether they
! fall within. If not, set unrealistic negative value to ignore
! these particular point sources.

        do is=1,Nsrc
          Isrc(is)=Isrc(is)-iSW_corn   ! Transl
          Jsrc(is)=Jsrc(is)-jSW_corn
          if ( Isrc(is).lt.iwest  .or. Isrc(is).gt.ieast  .or.
     &         Jsrc(is).lt.jsouth .or. Jsrc(is).gt.jnorth ) then
            Isrc(is)=-100
            Jsrc(is)=-100
            do itrc=1,NT
              Lsrc(is,itrc)=.false.
            enddo
          else
            write(*,'(2X,A,X,I4,X,A,X,I3,A,2(I4,A))')
     &         '--> PSOURCE No.',is,'is found in node',mynode,
     &         ' : (i,j)=(',Isrc(is),',',Jsrc(is),')'
          endif
        enddo
# endif

! Dsrc (Nsrc): direction of mass flux (0:xi-, 1:eta-, 2:s-direction)

        ierr=nf_inq_varid (ncid, 'Dsrc', varid)
        if (ierr .eq. nf_noerr) then
          ierr=nf_get_var_int (ncid, varid, Dsrc)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'Dsrc', frcname(1:lstr)
            goto 99                                      !--> ERROR
          endif
        else
          write(*,1) 'Dsrc id', frcname(1:lstr)
          goto 99                                        !--> ERROR
        endif

CR        fmt = '(6x,' / / snsrc / / 'i4)'
CR        mpi_master_only write(*,*) 'get_psrcs: Isrc'
CR        mpi_master_only write(*,fmt) (Isrc(i),i=1,Nsrc)
CR        mpi_master_only write(*,*) 'get_psrcs: Jsrc'
CR        mpi_master_only write(*,fmt) (Jsrc(i),i=1,Nsrc)
CR        mpi_master_only write(*,*) 'get_psrcs: Dsrc'
CR        mpi_master_only write(*,fmt) (Dsrc(i),i=1,Nsrc)

! Qshape (Nsrc,N)

        ierr=nf_inq_varid (ncid, 'Qshape', varid)
        if (ierr .eq. nf_noerr) then
          ierr=nf_get_var_FTYPE (ncid, varid, D)
          if (ierr .eq. nf_noerr) then
            Qshape=0.D0
            do is=1,Nsrc
              cff=0.D0; csum=0.D0
              do k=1,N
                Qshape(is,k)=D(is,k)
                cff=cff+Qshape(is,k)
              enddo
!              write(*,*) 'is= ',is , ' - Qshape = ',Qshape(is,:)
              if (cff.ne.0.D0) then
                csum=1./cff
              else
                csum=0.D0
              endif
              if (abs(cff-1.0).gt.1.D-6) then
                write(*,*) 'WARNING : get_psrcs :: Qshape is not ',
     &                     'normalized.  Correction has been made.'
                do k=1,N            ! normalize Qshape
                  Qshape(is,k)=Qshape(is,k)*csum
                enddo
              endif
!              write(*,*) 'is2= ',is , ' - Qshape2 = ',Qshape(is,:)

            enddo
          else
            write(*,1) 'Qshape', frcname(1:lstr)
            goto 99                                      !--> ERROR
          endif
        else
          write(*,1) 'Qshape', frcname(1:lstr)
          goto 99                                        !--> ERROR
        endif

!              write(*,*) ' - Qshape1 = ',Qshape(1,:)
!              write(*,*) ' - Qshape160 = ',Qshape(160,:)

CR
CR        mpi_master_only write(*,*) 'get_psrcs: Qshape'
CR        mpi_master_only write(sk,'(i7)') N
CR        fmt = '(6x,i4,' / / sk / / 'f6.3)'
CR        do i=1,Nsrc
CR          mpi_master_only write(*,fmt) i,(Qshape(i,k),k=1,N)
CR        enddo
CR
! Qbar id
        ierr=nf_inq_varid (ncid, 'Qbar', QbarG_id)
        if (ierr .ne. nf_noerr) then
          write(*,1) 'Qbar', frcname(1:lstr)
          goto 99                                        !--> ERROR
        endif

! Tsrc id
        do itrc=1,NT
          lvar=lenstr(vname(1,indxT+itrc-1))
          ierr=nf_inq_varid (ncid, vname(1,indxT+itrc-1)(1:lvar),
     &                                            TsrcG_id(itrc))
        
         write(*,*) 'get_psrc: Tname = ',vname(1,indxT+itrc-1)(1:lvar)

          if (ierr .ne. nf_noerr) then
            write(*,1) vname(1,indxT+itrc-1)(1:lvar), 
     &            frcname(1:lstr)
!            goto 99                                      !--> ERROR
          endif
        enddo


!!  Print diagnosis

c# define CD
CD        write(*,*)'[ get_psrcs diagnosis ]' MYID
CD        write(*,'(6x,a,i4)')'number of point sources, Nsrc = ',Nsrc
CD        fmt = '(6x,a,' / / snsrc / / 'i2)'
CD        write(*,fmt)'direction : ', (Dsrc(i),i=1,Nsrc)
CD        fmt = '(6x,3a,' / / snsrc / / 'i2)'
CD        do itrc=1,NT
CD          lvar=lenstr(vname(1,indxT+itrc-1))
CD          write(*,fmt) 'tracer ', vname(1,indxT+itrc-1)(1:lvar)
CD        enddo

! time
        ierr=nf_inq_varid (ncid, 'psrc_time', psrc_tid)
        if (ierr .eq. nf_noerr) then
          call set_cycle (ncid, psrc_tid, ntpsrc, psrc_cycle,
     &                    psrc_ncycle, psrc_rec, ierr)
          itpsrc=1
          psrc_time(1)=-1.E+20
          psrc_time(2)=-1.E+20
        else
          write(*,1) 'psrc_time', frcname(1:lstr)
          goto 99                                        !--> ERROR
        endif

!        write(*,*) 'nf_noerr2 = ',nf_noerr,'ierr2 = ',ierr

!        mpi_master_only write(*,*) 'wrt_ntpsrc=',ntpsrc
!        mpi_master_only write(*,*) 'wrt_psrc_rec=',psrc_rec
!        mpi_master_only write(*,*) 'wrt_psrc_cycle=',psrc_cycle
!        mpi_master_only write(*,*) 'wrt_psrc_ncycle=',psrc_ncycle
!        mpi_master_only write(*,*) 'wrt_time=',time
!        mpi_master_only write(*,*) 'wrt_psrc_time=',psrc_time(1)

        if (allocated(C)) deallocate (C)
        if (allocated(D)) deallocate (D)

      endif             ! iic.eq.ntstart
  1   format(1x,'WARNING: get_psrcs :: Cannot find variable ''',
     &                          A, ''' in file ''', A, '''.')

! Read data from the file: Check if model time is bounded by the past
!----- ---- ---- --- ----- and the future data times: if not, increment
! record and cycling indices, flip time index and read a new portion of
! the data. Repeat until model time falls between the two data times.
CR
CR        write(*,*) 'get_psrcs: ntpsrc=',ntpsrc
CR
!      write(*,*) 'get_psrcs: itpsrc=',itpsrc
!      write(*,*) 'get_psrcs: time+0.5*dt=',time+0.5*dt
!      write(*,*) 'wrt_psrc_time1 = ', psrc_time*sec2day
!            QbarG(1:Nsrc,1:2) = 1.0
      do while (psrc_time(itpsrc).lt.time+0.5*dt  .and.
     &                                    ierr.eq.nf_noerr)
!      write(*,*) 'get_psrcs: itpsrc2=',itpsrc
        call advance_cycle (psrc_cycle, ntpsrc,
     &                         psrc_ncycle, psrc_rec, ierr)

        if (ierr.eq.nf_noerr) then
          ierr=nf_get_var1_FTYPE (ncid, psrc_tid, psrc_rec, cff)
          if (ierr .eq. nf_noerr) then
            itpsrc=min(3-itpsrc,ntpsrc)
!            itpsrc=min(itpsrc,ntpsrc)
            psrc_time(itpsrc)=psrc_cycle*psrc_ncycle + cff*day2sec
!            write(*,*) 'wrt_psrc_time 2 =  ' , psrc_time(itpsrc)*sec2day
          write(*,*) 'get_psrcs: itpsrc2=',itpsrc , ' - psrc_time  = ',psrc_time(itpsrc)

            start(2)=1
            count(2)=Nsrc
            start(1)=psrc_rec
            count(1)=1

!          write(*,*) 'start = ',start
!          write(*,*) 'count = ',count
!          write(*,*) 'itpsrc_qbarg = ',itpsrc
!          write(*,*) 'cff = ',cff
! Qbar
!            var1d = 3.0
!           ierr=nf_get_vara_FTYPE (ncid, QbarG_id, start, count,
!    &                                           QbarG(1,itpsrc))
            ierr=nf_get_vara_FTYPE (ncid, QbarG_id, start, count,
     &                                           var1d)
!            write(*,*) 'var1d = ', var1d
            QbarG(:,itpsrc)=var1d
!             write(*,*) 'QbarG = ', QbarG
!            stop 
!            ierr=nf_get_vara_FTYPE (ncid, QbarG_id, start, count,
!     &                                           QbarG)

!          ierr=nf_get_vara_FTYPE(ncid,QbarG_id,start,Nsrc,
!     &         QbarG(1:Nsrc,itpsrc))
!          ierr=nf_get_var_FTYPE (ncid, QbarG_id,Qbar)
!            write(*,*) 'get_psrc:: QbarG = ',Qbar

           if (ierr .ne. nf_noerr) then
              write(*,2) 'Qbar', psrc_rec, nf_strerror(ierr)
              goto 99                                    !--> ERROR
            endif

!          write(snsrc,'(i7)') Nsrc
!          write(*,*) 'get_psrcs: QbarG: ', QbarG(i,itpsrc)
!        fmt = '(6x,' / / snsrc / / 'f6.2)'
!        write(*,fmt) (QbarG(i,itpsrc),i=1,Nsrc)
!! Tsrc
!            do itrc=1,Npas-2
!              lvar=lenstr(vname(1,indxT+itrc-1))
!              ierr=nf_get_vara_FTYPE (ncid, TsrcG_id(itrc), start,
!     &                                count,  TsrcG(1,itpsrc,itrc))
!              if (ierr .ne. nf_noerr) then
!!                write(*,2) vname(1,indxT+itrc-1)(1:lvar), psrc_rec, 
!!     &                                            nf_strerror(ierr)
!!                goto 99                                  !--> ERROR
!            write(*,1) vname(1,indxT+itrc-1)(1:lvar),
!     &            frcname(1:lstr)
!              endif
! 
!!              write(*,*) 'itrc = ',itrc
!!CR
!!CR        write(*,*) 'get_psrcs: ',vname(1,indxT+itrc-1)
!!CR        fmt = '(6x,' / / snsrc / / 'f6.2)'
!!CR        write(*,fmt) (TsrcG(i,itpsrc,itrc),i=1,Nsrc)
!!CR

!           enddo

!             write(*,*) 'QbarG1 = ', QbarG(:,1)
!             write(*,*) 'QbarG2 = ', QbarG(:,2)
          mpi_nonexit_warn write(*,'(3x,A,3x,A,F12.4,2(1x,A,I4))')
     &                'get_psrcs :: read point source file for ',
     &                'psrc_time =', cff, 'rec =',psrc_rec MYID
c              write(*,'(6x,2A,F12.4,2(1x,A,I4))') 'get_psrcs',
c     &                '-- read point source file    for psrc_time =',
c     &                   cff, 'rec =', psrc_rec MYID
          else
            write(*,2) 'psrc_time', psrc_rec,  nf_strerror(ierr)
          endif
        else
          write(*,'(/2(1x,A,I4)/12x,3A/12x,A,G12.4,A,G12.4/)')
     &          '### ERROR: get_psrcs :: Requested time record ',
     &           psrc_rec,    'exeeds the last record',   ntpsrc,
     &          'available in netCDF file ''',  frcname(1:lstr),
     &          '''',  'tdays = ',   tdays,  '   while the last ',
     &         'available psrc_time =', psrc_time(itpsrc)*sec2day
        endif
      enddo

! DO The temporal interpolation abd extract Qbar(:) vector to replace
! QbarG(:,1) and QbarG(:,2)
      it1=3-itpsrc             ! Always set synchro_flag, regardless
      it2=itpsrc                ! whether the boundary is needed or not:
      cff=time+0.5*dt          ! this is necessary to make sure that
      cff1=psrc_time(it2)-cff   ! running out of data records condition
      cff2=cff-psrc_time(it1)   ! is the same for all MPI nodes.

      if (cff1.ge.0. .and. cff2.ge.0.) then
!               write(*,*) 'I am here psrc'
        if (cff1.lt.dt) synchro_flag=.true.
c*        if (ZEROTH_TILE .and. cff1.lt.dt) synchro_flag=.TRUE.

        cff=1./(cff1+cff2)
        cff1=cff1*cff
        cff2=cff2*cff

        do is=1,Nsrc
          Qbar(is)=cff1*QbarG(is,it1)+cff2*QbarG(is,it2)
!            write(*,*) 'Qbar = ',QbarG(is,it1)
c*          if (QbarG(is,it1) .eq. 0. .or. QbarG(is,it2) .eq.0) then
c*              Qbar(is) = 0.  ! makes sure set_psource doesn't
c*          endif              ! interpolate more flow than you want
          do k=1,N
!            write(*,*) 'Qbar = ',Qbar(is)
!            write(*,*) 'Qshape = ',Qshape(is,k)
            Qsrc(is,k)=Qbar(is)*Qshape(is,k)
          enddo
!            write(*,*) 'Qbar1 = ',QbarG(1,it1)
!            write(*,*) 'Qbar2 = ',QbarG(1,it2)
!            write(*,*) 'Qshape = ',Qshape(1,:)
!            write(*,*) 'Qsrc1 = ',Qsrc(1,:)
!            write(*,*) 'Qsrc160 = ',Qsrc(160,:)
          do itrc=1,NT
!            if (Lsrc(is,itrc)) then
              Tsrc0(is,itrc)=cff1*TsrcG(is,it1,itrc)
     &                     + cff2*TsrcG(is,it2,itrc)
              do k=1,N
                 Tsrc(is,k,itrc)=Tsrc0(is,itrc)
              enddo
!            endif
          enddo

        enddo
!  Unable to set-up Qbar and Tsrc0:
!  Complain about the error and signal to quit.
      else
!        if (ZEROTH_TILE) then
        if (cff1.lt.0. .or. cff2.lt.0.) then
!           write(*,*) 'wrt_psrc_time =  ' , psrc_time(itpsrc)*sec2day
            write(*,'(/1x,3A/3(1x,A,F16.10)/)')      '### WARNING: ',
     &    'set_psource_tile :: Current model time is outside bounds ',
     &    'of ''psrc_time''.', 'psrc_tstart =', psrc_time(it1)*sec2day,
     &                         'tdays =',       tdays,
     &                         'psrc_tend =',   psrc_time(it2)*sec2day
          may_day_flag=2
        endif
        ierr=ierr+1
      endif


  2   format(/1x,'### ERROR: get_psrcs :: Cannot read variable ''',
     &                                A, ''' for rec =', I4/12x,A/)

      return

  99  may_day_flag=2

      return
      end


#else
      subroutine get_psrcs_empty
      end
#endif /* PSOURCE && !ANA_PSOURCE */
