#include "cppdefs.h"
#if defined PSOURCE && !defined ANA_PSOURCE

      subroutine get_psrcs (ierr)
!      subroutine get_psrcs (istr,iend,jstr,jend,ierr)

! Read in point sources of flow and tracer at the appropriate
! time from psource NetCDF files.  Written by Eileen Deng-Idica
! (05-03-2007) & modified by Yusuke Uchiyama (06-20-2011)
! modified by Faycal Kessouri (08-2018) : read point-sources of 
! pollution inputs and rivers from a NetCDF file
      implicit none
# include "param.h"
# include "forces.h"
# include "sources.h"
# include "scalars.h"
# include "netcdf.inc"
# include "ncvars.h"
# include "ocean3d.h"
# include "grid.h"
# ifdef MPI
#  include "hidden_mpi_vars.h"
# endif

      real cff, csum, cff1,cff2
      integer ierr, lstr,lvar,lenstr,ifrc,Npas
      integer istr,iend,jstr,jend
      integer is, itrc, ncid, varid, i,j,k, it1,it2
#ifdef MPI
     &                        , i1,i2,j1,j2
#endif
      integer, parameter :: pdims=2
      integer            :: start(pdims), count(pdims)
      real,    dimension(:,:), allocatable :: D
      integer, dimension(:,:), allocatable :: C
      character(len=7)  snsrc, sk
      character(len=64) fmt
      character*(max_name_size) frcname

! Initialization: Inquire about the contents of psource NetCDF file:
!================  variables and dimensions. Check for consistency.

      if (may_day_flag.ne.0) return      !-->  EXIT

! calculate volume


!      do j=jstr,jend
!         do i=istr,iend
!#ifdef MASKING
!           cffa(i,j)=rmask(i,j)/(pm(i,j)*pn(i,j))  ! <-- grid box area
!#else
!           cffa(i,j)=1./(pm(i,j)*pn(i,j))
!#endif
!           do k=1,N
!              Vz(i,j,k)=cffa(i,j)*Hz(i,j,k) ! volume
!           enddo
!         enddo
!      enddo
!
!#ifdef MASKING
!           cffa=rmask(i,j)/(pm(i,j)*pn(i,j))  ! <-- grid box area
!#else
!           cffa=1./(pm(i,j)*pn(i,j))
!#endif

      lstr=lenstr(frcname)
      ncid=ncidfpsrc
      ierr=nf_noerr

      if (iic.eq.ntstart) then

# if defined BULK_FLUX
      ifrc=max(ncidfpsrc,1)
 100  ncid=ncidbulk(ifrc)
      ierr=nf_noerr
      frcname=blkfile(ifrc)
# else
      ifrc=max(ncidfpsrc,1)
 100  ncid=ncidfrc(ifrc)
      ierr=nf_noerr
      frcname=frcfile(ifrc)
# endif

!      if (iic.eq.ntstart) then
        lstr=lenstr(frcname)
        if (ncid.eq.-1) then
# if defined BULK_FLUX
          ierr=nf_open(frcname(1:lstr), nf_nowrite, ncid)
          if (ierr.eq.nf_noerr) then
            ncidbulk(ifrc)=ncid
          else
            write(*,'(/1x,4A/)') '### ERROR ',
     &                   'in get_psrcs: can not open netCDF file ''',
     &                                       frcname(1:lstr), '''.'
          endif
# else

          ierr=nf_open(frcname(1:lstr), nf_nowrite, ncid)
          if (ierr.eq.nf_noerr) then
            ncidfrc(ifrc)=ncid
          else
            write(*,'(/1x,4A/)') '### ERROR ',
     &                   'in get_psrcs: can not open netCDF file ''',
     &                                       frcname(1:lstr), '''.'
          endif
# endif

        endif
!      endif
      write(*,*) 'source_file_name = ',frcname(1:lstr)

! Nsrc: number of point sources
        ierr=nf_inq_dimid (ncid, 'Nsrc', varid)
        if (ierr .eq. nf_noerr) then
          ierr=nf_inq_dimlen (ncid, varid, Nsrc)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'Nsrc', frcname(1:lstr)
            goto 99                                      !--> ERROR
          else
            if (Nsrc.gt.Msrc) then
              write(*,*) 'FATAL ERROR: get_psrcs :: Found Nsrc > Msrc'
              goto 99                                    !--> ERROR
            endif
          endif
        else
          write(*,1) 'Nsrc', frcname(1:lstr)
          goto 99                                        !--> ERROR
        endif
!                  Nsrc = 1

        write(snsrc,'(i7)') Nsrc
        if (.not.allocated(C)) allocate (C(Nsrc,NT))
        if (.not.allocated(D)) allocate (D(Nsrc,N))

             write(*,*) 'get_psrc:: Nsrc = ',Nsrc

! Npas: number of tracers
        ierr=nf_inq_dimid (ncid, 'Npas', varid)
        if (ierr .eq. nf_noerr) then
          ierr=nf_inq_dimlen (ncid, varid, Npas)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'Npas', frcname(1:lstr)
            goto 99                                      !--> ERROR
          else
            if (Npas.gt.NT) then
              write(*,*) 'FATAL ERROR: get_psrcs :: Found Npas > NT'
              goto 99                                    !--> ERROR
            endif
          endif
        else
          write(*,1) 'Npas', frcname(1:lstr)
          goto 99                                        !--> ERROR
        endif
!        write(*,*) 'Nsrc = ',Nsrc ,' Msrc = ',Msrc

! Lsrc (Nsrc,itrc): logical switch
!        Lsrc =.false.
!        ierr=nf_inq_varid (ncid, 'Lsrc', varid)
!        if (ierr .eq. nf_noerr) then
!          ierr=nf_get_var_int (ncid, varid, C)
!          if (ierr .ne. nf_noerr) then
!            write(*,1) 'Lsrc', frcname(1:lstr)
!            goto 99                                      !--> ERROR
!          endif
!          do itrc=1,NT
!            do is=1,Nsrc
!              if (C(is,itrc).eq.1) Lsrc(is,itrc)=.true.
!            enddo
!          enddo
!        else
!          write(*,1) 'Lsrc id', frcname(1:lstr)
!          goto 99                                        !--> ERROR
!        endif

! Isrc & Jsrc (Nsrc): point source locations relative to the global grid
        ierr=nf_inq_varid (ncid, 'Jsrc', varid)
        if (ierr .eq. nf_noerr) then
          ierr=nf_get_var_int (ncid, varid, Isrc)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'Isrc', frcname(1:lstr)
            goto 99                                      !--> ERROR
          endif
        else
          write(*,1) 'Isrc id', frcname(1:lstr)
          goto 99                                        !--> ERROR
        endif

        ierr=nf_inq_varid (ncid, 'Isrc', varid)
        if (ierr .eq. nf_noerr) then
          ierr=nf_get_var_int (ncid, varid, Jsrc)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'Jsrc', frcname(1:lstr)
            goto 99                                      !--> ERROR
          endif
        else
          write(*,1) 'Jsrc id', frcname(1:lstr)
          goto 99                                        !--> ERROR
        endif

! Initially indices "Isrc,Jsrc" are assumed to be defined relative
! to the absolute physical grid. Translate them into would be local
! indices within the current MPI-subdomain and check whether they
! fall within. If not, set unrealistic negative value to ignore
! these particular point sources.

!       write(*,*) 'KESF1 :: Isrc=', Isrc
!       write(*,*) 'KESF1 :: Jsrc=', Jsrc



!        do is=1,Nsrc
        is = 30
#  ifdef MPI
!        if (iminmpi.LE.Isrc(is) .AND. Isrc(is).LE.imaxmpi .AND.
!     &       jminmpi.LE.Jsrc(is) .AND. Jsrc(is).LE.jmaxmpi) then
!          Isrc_mpi(is,mynode)=Isrc(is)-iminmpi+1
!          Jsrc_mpi(is,mynode)=Jsrc(is)-jminmpi+1
!        else
!          Isrc_mpi(is,mynode)=-1
!          Jsrc_mpi(is,mynode)=-1
!        endif

!            write(*,*) 'is= ',is,' ii = ',Isrc(is)
!           write(*,*) 'is= ',is,' jj = ',Jsrc(is)

!          IIsrc(is)=Isrc(is)
!          JJsrc(is)=Jsrc(is)

!          Isrc(is)=Isrc(is)-iSW_corn   ! Transl
!          Jsrc(is)=Jsrc(is)-jSW_corn
          
!            write(*,*) 'is= ',is,' ii2 = ',Isrc(is)
!            write(*,*) 'is= ',is,' jj2 = ',Jsrc(is)

!          if ( Isrc(is).lt.iwest  .or. Isrc(is).gt.ieast  .or.
!     &         Jsrc(is).lt.jsouth .or. Jsrc(is).gt.jnorth ) then

          if ( (Isrc(is)-iSW_corn).lt.iwest  .or. (Isrc(is)-iSW_corn).gt.ieast  .or.
     &         (Jsrc(is)-jSW_corn).lt.jsouth .or. (Jsrc(is)-jSW_corn).gt.jnorth ) then

            Isrc(is)=-100
            Jsrc(is)=-100
!            do itrc=1,NT
!              Lsrc(is,itrc)=.false.
!            enddo
          else
            write(*,'(2X,A,X,I4,X,A,X,I3,A,2(I4,A))')
     &         '--> PSOURCE No.',is,'is found in node',mynode,
     &         ' : (i,j)=(',Isrc(is),',',Jsrc(is),')'
                if (mynode>0) then
                        Mynnd(is) = mynode
                        indii(is) = Isrc(is)
                        indjj(is) = Jsrc(is)

                       write(*,*) 'KESF :: Isrc=', Isrc(is)
                       write(*,*) 'KESF :: Jsrc=', indjj(is)
                       write(*,*) 'KESF :: Mynnd=', Mynnd(is)

                endif
!     &         ' : (i,j)=(',(Isrc(is)-iSW_corn),',',(Jsrc(is)-jSW_corn),')'
!                indii(is) = IIsrc(is)
!                indjj(is) = JJsrc(is)

!     &         ' : (i,j)=(',Isrc_mpi(is,mynode),',',Jsrc_mpi(is,mynode),')'
!          write(*,*) 'KESF :: Isrc=', Isrc(is)
!       write(*,*) 'KESF :: Jsrc=', indjj(is)
!       write(*,*) 'KESF :: Mynnd=', Mynnd(is)
!                stop
          endif
# endif
!        enddo ! is

!       write(*,*) 'KESF :: Isrc=', Isrc(is)
!       write(*,*) 'KESF :: Jsrc=', indjj(is)
!       write(*,*) 'KESF :: Mynnd=', Mynnd(is)

!                stop
! Dsrc (Nsrc): direction of mass flux (0:xi-, 1:eta-, 2:s-direction)
        ierr=nf_inq_varid (ncid, 'Dsrc', varid)
        if (ierr .eq. nf_noerr) then
          ierr=nf_get_var_int (ncid, varid, Dsrc)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'Dsrc', frcname(1:lstr)
            goto 99                                      !--> ERROR
          endif
        else
          write(*,1) 'Dsrc id', frcname(1:lstr)
          goto 99                                        !--> ERROR
        endif

! Qshape (Nsrc,N)
        ierr=nf_inq_varid (ncid, 'Qshape', varid)
        if (ierr .eq. nf_noerr) then
          ierr=nf_get_var_FTYPE (ncid, varid, D)
          if (ierr .eq. nf_noerr) then
            Qshape=0.D0
!            do is=1,Nsrc
             is =30
              cff=0.D0; csum=0.D0
              do k=1,N
                Qshape(is,k)=D(is,k)
                cff=cff+Qshape(is,k)
              enddo
!              write(*,*) 'is= ',is , ' - Qshape = ',Qshape(is,:)
              if (cff.ne.0.D0) then
                csum=1./cff
              else
                csum=0.D0
              endif
              if (abs(cff-1.0).gt.1.D-6) then
                write(*,*) 'WARNING : get_psrcs :: Qshape is not ',
     &                     'normalized.  Correction has been made.'
                do k=1,N            ! normalize Qshape
                  Qshape(is,k)=Qshape(is,k)*csum
                enddo
              endif
!            enddo   ! is
          else
            write(*,1) 'Qshape', frcname(1:lstr)
            goto 99                                      !--> ERROR
          endif
        else
          write(*,1) 'Qshape', frcname(1:lstr)
          goto 99                                        !--> ERROR
        endif
!              write(*,*) ' - Qshape1 = ',Qshape(is,:)
!              write(*,*) ' - Qshape2 = ',D(is,:)
!                stop

!              write(*,*) ' - Qshape160 = ',Qshape(160,:)

! Qbar id
        ierr=nf_inq_varid (ncid, 'Qbar', QbarG_id)
        if (ierr .ne. nf_noerr) then
          write(*,1) 'Qbar', frcname(1:lstr)
          goto 99                                        !--> ERROR
        endif


! Tsrc id
!        do itrc=1,Npas-2
!          lvar=lenstr(vname(1,indxT+itrc-1))
!          ierr=nf_inq_varid (ncid, vname(1,indxT+itrc-1)(1:lvar),
!     &                                            TsrcG_id(itrc))
!!         write(*,*) 'get_psrc: Tname = ',vname(1,indxT+itrc-1)(1:lvar)
!          if (ierr .ne. nf_noerr) then
!            write(*,1) vname(1,indxT+itrc-1)(1:lvar), 
!     &            frcname(1:lstr)
!!            goto 99                                      !--> ERROR
!          endif
!        enddo

! var id
          ierr=nf_inq_varid (ncid, 'temp',tempG_id)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'temp'
          endif

          ierr=nf_inq_varid (ncid, 'salt',saltG_id)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'salt'
          endif

          ierr=nf_inq_varid (ncid, 'PO4',PO4G_id)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'PO4'
          endif

          ierr=nf_inq_varid (ncid, 'NO3',NO3G_id)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'NO3'
          endif

          ierr=nf_inq_varid (ncid, 'NH4',NH4G_id)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'NH4'
          endif

          ierr=nf_inq_varid (ncid, 'Fe',FEG_id)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'FE'
          endif

          ierr=nf_inq_varid (ncid, 'O2',O2G_id)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'O2'
          endif

          ierr=nf_inq_varid (ncid, 'DIC',DICG_id)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'DIC'
          endif

          ierr=nf_inq_varid (ncid, 'Alk',ALKG_id)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'ALK'
          endif

          ierr=nf_inq_varid (ncid, 'DOC',DOCG_id)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'DOC'
          endif

          ierr=nf_inq_varid (ncid, 'DON',DONG_id)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'DON'
          endif

          ierr=nf_inq_varid (ncid, 'DOP',DOPG_id)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'DOP'
          endif

          ierr=nf_inq_varid (ncid, 'NO2',NO2G_id)
          if (ierr .ne. nf_noerr) then
            write(*,1) 'NO2'
          endif

!!  Print diagnosis

! time
        ierr=nf_inq_varid (ncid, 'psrc_time', psrc_tid)
        if (ierr .eq. nf_noerr) then
          call set_cycle (ncid, psrc_tid, ntpsrc, psrc_cycle,
     &                    psrc_ncycle, psrc_rec, ierr)
          itpsrc=1
          psrc_time(1)=-1.E+20
          psrc_time(2)=-1.E+20
        else
          write(*,1) 'psrc_time', frcname(1:lstr)
          goto 99                                        !--> ERROR
        endif

        if (allocated(C)) deallocate (C)
        if (allocated(D)) deallocate (D)

      endif             ! iic.eq.ntstart
! 1   format(1x,'WARNING: get_psrcs :: Cannot find variable ''',
!    &                          A, ''' in file ''', A, '''.')

! Read data from the file: Check if model time is bounded by the past
!----- ---- ---- --- ----- and the future data times: if not, increment
! record and cycling indices, flip time index and read a new portion of
! the data. Repeat until model time falls between the two data times.
!      write(*,*) 'get_psrcs: itpsrc=',itpsrc
!      write(*,*) 'get_psrcs: time+0.5*dt=',time+0.5*dt
!      write(*,*) 'wrt_psrc_time1 = ', psrc_time*sec2day
      do while (psrc_time(itpsrc).lt.time+0.5*dt  .and.
     &                                    ierr.eq.nf_noerr)
        call advance_cycle (psrc_cycle, ntpsrc,
     &                         psrc_ncycle, psrc_rec, ierr)
        if (ierr.eq.nf_noerr) then
          ierr=nf_get_var1_FTYPE (ncid, psrc_tid, psrc_rec, cff)
          if (ierr .eq. nf_noerr) then
            itpsrc=min(3-itpsrc,ntpsrc)
            psrc_time(itpsrc)=psrc_cycle*psrc_ncycle + cff*day2sec

            start(2)=1
            count(2)=Nsrc
            start(1)=psrc_rec
            count(1)=1

! Qbar
            ierr=nf_get_vara_FTYPE (ncid, QbarG_id, start, count,
     &                                           QbarG(:,itpsrc))
           if (ierr .ne. nf_noerr) then
              write(*,2) 'Qbar', psrc_rec, nf_strerror(ierr)
              goto 99                                    !--> ERROR
           endif
!! Tsrc
!            do itrc=1,NT
!              lvar=lenstr(vname(1,indxT+itrc-1))
!              ierr=nf_get_vara_FTYPE (ncid, TsrcG_id(itrc), start,
!     &                                count,  TsrcG(:,itpsrc,itrc))
!              write(*,*) 'vname = ',vname(1,indxT+itrc-1)(1:lvar)
!              if (ierr .ne. nf_noerr) then
!                write(*,1) vname(1,indxT+itrc-1)(1:lvar), psrc_rec,
!     &                                            nf_strerror(ierr)
!!                goto 99                                  !--> ERROR
!              endif
!            enddo

             ierr=nf_get_vara_FTYPE (ncid, tempG_id, start, count,
     &                                           tempG(:,itpsrc))

             ierr=nf_get_vara_FTYPE (ncid, saltG_id, start, count,
     &                                           saltG(:,itpsrc))

             ierr=nf_get_vara_FTYPE (ncid, PO4G_id, start, count,
     &                                           PO4G(:,itpsrc))

             ierr=nf_get_vara_FTYPE (ncid, NO3G_id, start, count,
     &                                           NO3G(:,itpsrc))

             ierr=nf_get_vara_FTYPE (ncid, NH4G_id, start, count,
     &                                           NH4G(:,itpsrc))

             ierr=nf_get_vara_FTYPE (ncid, FEG_id, start, count,
     &                                           FEG(:,itpsrc))

             ierr=nf_get_vara_FTYPE (ncid, O2G_id, start, count,
     &                                           O2G(:,itpsrc))

             ierr=nf_get_vara_FTYPE (ncid, DICG_id, start, count,
     &                                           DICG(:,itpsrc))

             ierr=nf_get_vara_FTYPE (ncid, ALKG_id, start, count,
     &                                           ALKG(:,itpsrc))

             ierr=nf_get_vara_FTYPE (ncid, DOCG_id, start, count,
     &                                           DOCG(:,itpsrc))

             ierr=nf_get_vara_FTYPE (ncid, DONG_id, start, count,
     &                                           NO3G(:,itpsrc))

             ierr=nf_get_vara_FTYPE (ncid, DOPG_id, start, count,
     &                                           DOPG(:,itpsrc))

             ierr=nf_get_vara_FTYPE (ncid, NO2G_id, start, count,
     &                                           NO2G(:,itpsrc))
 
!            do itrc=1,Npas-2
!!            ierr=nf_get_vara_FTYPE (ncid, TsrcG_id(itrc), start, count,
!!     &                                           TsrcG(:,itpsrc,itrc))
!             if (ierr .ne. nf_noerr) then
!               write(*,1) vname(1,indxT+itrc-1)(1:lvar),
!     &            frcname(1:lstr)
!             endif

!              write(*,*) 'itrc = ',itrc , ' - variable= ', vname(1,indxT+itrc-1)
!            enddo

!! ERROR MESSAGES HERE
          mpi_nonexit_warn write(*,'(3x,A,3x,A,F12.4,2(1x,A,I4))')
     &                'get_psrcs :: read point source file for ',
     &                'psrc_time =', cff, 'rec =',psrc_rec MYID
          else
            write(*,2) 'psrc_time', psrc_rec,  nf_strerror(ierr)
          endif
        else
          write(*,'(/2(1x,A,I4)/12x,3A/12x,A,G12.4,A,G12.4/)')
     &          '### ERROR: get_psrcs :: Requested time record ',
     &           psrc_rec,    'exeeds the last record',   ntpsrc,
     &          'available in netCDF file ''',  frcname(1:lstr),
     &          '''',  'tdays = ',   tdays,  '   while the last ',
     &         'available psrc_time =', psrc_time(itpsrc)*sec2day
        endif
      enddo

!             write(*,*) 'NH4G1 = ', NH4G(:,1)
!             write(*,*) 'NH4G2 = ', NH4G(:,2)

!!!!!! LAST PART !!!!!!!!!!!!!!!!!
!!!!!! TIME INTERPOLATION  !!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

      it1=3-itpsrc             ! Always set synchro_flag, regardless
      it2=itpsrc                ! whether the boundary is needed or not:
      cff=time+0.5*dt          ! this is necessary to make sure that
      cff1=psrc_time(it2)-cff   ! running out of data records condition
      cff2=cff-psrc_time(it1)   ! is the same for all MPI nodes.

      if (cff1.ge.0. .and. cff2.ge.0.) then
        if (cff1.lt.dt) synchro_flag=.true.
        cff=1./(cff1+cff2)
        cff1=cff1*cff
        cff2=cff2*cff

!        do is=1,Nsrc
        is = 30
!# ifdef MPI
!        i=Isrc_mpi(is,mynode)
!        j=Jsrc_mpi(is,mynode)
!# else
!        i=Isrc(is)
!        j=Jsrc(is)
!# endif


!        i=Isrc(is)           ! with river runoff simulations.
!        j=Jsrc(is)
!!          if ( Isrc(is).lt.iwest  .or. Isrc(is).gt.ieast  .or.
!     &         Jsrc(is).lt.jsouth .or. Jsrc(is).gt.jnorth ) then
!
!        if (iminmpi.LE.Isrc(is) .AND. Isrc(is).LE.imaxmpi .AND.
!     &       jminmpi.LE.Jsrc(is) .AND. Jsrc(is).LE.jmaxmpi) then
!          Isrc_mpi(is,mynode)=Isrc(is)-iminmpi+1
!          Jsrc_mpi(is,mynode)=Jsrc(is)-jminmpi+1


!          write(*,*) 'volume= ',Vz(Isrc(is),Jsrc(is)  , :)
          Qbar(is)=cff1*QbarG(is,it1)+cff2*QbarG(is,it2)
          Qtemp(is)=cff1*tempG(is,it1)+cff2*tempG(is,it2)
          Qsalt(is)=cff1*saltG(is,it1)+cff2*saltG(is,it2)
          QPO4(is)=cff1*PO4G(is,it1)+cff2*PO4G(is,it2)
          QNO3(is)=cff1*NO3G(is,it1)+cff2*NO3G(is,it2)
          QNH4(is)=cff1*NH4G(is,it1)+cff2*NH4G(is,it2)
          QFE(is)=cff1*FEG(is,it1)+cff2*FEG(is,it2)
          QO2(is)=cff1*O2G(is,it1)+cff2*O2G(is,it2)
          QDIC(is)=cff1*DICG(is,it1)+cff2*DICG(is,it2)
          QALK(is)=cff1*ALKG(is,it1)+cff2*ALKG(is,it2)
          QDOC(is)=cff1*DOCG(is,it1)+cff2*DOCG(is,it2)
          QDON(is)=cff1*DONG(is,it1)+cff2*DONG(is,it2)
          QDOP(is)=cff1*DOPG(is,it1)+cff2*DOPG(is,it2)
          QNO2(is)=cff1*NO2G(is,it1)+cff2*NO2G(is,it2)
!          write(*,*) 'isQsrc= ',is,' - Qbar= ',Qbar(is)
!          write(*,*) 'isQsrc= ',is,' - QNH4= ',QNH4(is)
!          write(*,*) 'isQsrc= ',is,' - Dsrc= ',Dsrc(is)
!          write(*,*) 'QNH4= ',QNH4(is)

!          do itrc=1,NT
!              Tsrc0(is,itrc)=cff1*TsrcG(is,it1,itrc)
!     &                     + cff2*TsrcG(is,it2,itrc)
!          enddo

!          endif
!        enddo ! is

!        do is=1,Nsrc
        is = 30
        i=Isrc(is)           ! with river runoff simulations.
        j=Jsrc(is)
          if ( Isrc(is).lt.iwest  .or. Isrc(is).gt.ieast  .or.
     &         Jsrc(is).lt.jsouth .or. Jsrc(is).gt.jnorth ) then
!          write(*,*) 'KESF_is = ',is
          do k=1,N
            Qsrc(is,k)=   Qbar(is)*Qshape(is,k)
            Qsrctemp(is,k)=Qtemp(is)
            Qsrcsalt(is,k)=Qsalt(is)
            QsrcPO4(is,k)=QPO4(is)
            QsrcNO3(is,k)=QNO3(is)
            QsrcNH4(is,k)=QNH4(is)
            QsrcFE(is,k)=QFE(is)
            QsrcO2(is,k)=QO2(is)
            QsrcDIC(is,k)=QDIC(is)
            QsrcALK(is,k)=QALK(is)
            QsrcDOC(is,k)=QDOC(is)
            QsrcDON(is,k)=QDON(is)
            QsrcDOP(is,k)=QDOP(is)
            QsrcNO2(is,k)=QNO2(is)

            do itrc=1,NT
                 Tsrc(is,k,itrc)=Tsrc0(is,itrc)
                 Tsrc(is,k,itemp)=Qsrctemp(is,k)
                 Tsrc(is,k,isalt)=Qsrcsalt(is,k)
                 Tsrc(is,k,iPO4)=QsrcPO4(is,k)
                 Tsrc(is,k,iNO3)=QsrcNO3(is,k)
                 Tsrc(is,k,iNH4)=QsrcNH4(is,k)
                 Tsrc(is,k,iFE)=QsrcFE(is,k)
                 Tsrc(is,k,iO2)=QsrcO2(is,k)
                 Tsrc(is,k,iDIC)=QsrcDIC(is,k)
                 Tsrc(is,k,iAlk)=QsrcALK(is,k)
                 Tsrc(is,k,iDOC)=QsrcDOC(is,k)
                 Tsrc(is,k,iDON)=QsrcDON(is,k)
                 Tsrc(is,k,iDOP)=QsrcDOP(is,k)
                 Tsrc(is,k,iNO2)=QsrcNO2(is,k)
            enddo

          enddo
          endif
!!          write(*,*) 'isQsrc= ',is,' - Qsrc= ',Qsrc(is,:)
!            write(*,*) 'Qis= ',is,' - QQsrc = ',Qsrc(is,:)
!            write(*,*) 'Qis= ',is,' - QQsrctemp = ',Qsrctemp(is,30)
!            write(*,*) 'Qis= ',is,' - QQsrcNH4 = ',QsrcNH4(is,30)
!            write(*,*) 'Qis= ',is,' - QTsrc1 = ',Tsrc(is,30,1)
!            write(*,*) 'Qis= ',is,' - QTsrc2 = ',Tsrc(is,30,2)
!            write(*,*) 'Qis= ',is,' - QTsrc3 = ',Tsrc(is,30,3)
!            write(*,*) 'Qis= ',is,' - QTsrc4 = ',Tsrc(is,30,4)
!            write(*,*) 'Qis= ',is,' - QTsrc5 = ',Tsrc(is,30,5)
!            write(*,*) 'Qis= ',is,' - QTsrc6 = ',Tsrc(is,30,6)
!            write(*,*) 'Qis= ',is,' - QTsrc7 = ',Tsrc(is,30,7)
!            write(*,*) 'Qis= ',is,' - QTsrc8 = ',Tsrc(is,30,8)
!            write(*,*) 'Qis= ',is,' - QTsrc9 = ',Tsrc(is,30,9)
!            write(*,*) 'Qis= ',is,' - QTsrc10 = ',Tsrc(is,30,10)
!            write(*,*) 'Qis= ',is,' - QTsrc11 = ',Tsrc(is,30,11)
!            write(*,*) 'Qis= ',is,' - QTsrc12 = ',Tsrc(is,30,12)
!            write(*,*) 'Qis= ',is,' - QDsrc= ',Dsrc(is)
!        enddo ! is



!      do is=1,Nsrc           ! Apply point mass sources associated
!        i=Isrc(is)           ! with river runoff simulations.
!        j=Jsrc(is)
!          if ( Isrc(is).lt.iwest  .or. Isrc(is).gt.ieast  .or.
!     &         Jsrc(is).lt.jsouth .or. Jsrc(is).gt.jnorth ) then
!            do k=1,N
!            QpsrcPO4_3D(i,j,k) = QsrcPO4(is,k)
!            QpsrcNO3_3D(i,j,k) = QsrcNO3(is,k)
!            QpsrcNH4_3D(i,j,k) = QsrcNH4(is,k)
!            QpsrcFE_3D(i,j,k) = QsrcFE(is,k)
!            QpsrcO2_3D(i,j,k) = QsrcO2(is,k)
!            QpsrcDIC_3D(i,j,k) = QsrcDIC(is,k)
!            QpsrcALK_3D(i,j,k) = QsrcALK(is,k)
!            QpsrcDOC_3D(i,j,k) = QsrcDOC(is,k)
!            QpsrcDON_3D(i,j,k) = QsrcDON(is,k)
!            QpsrcDOP_3D(i,j,k) = QsrcDOP(is,k)
!            QpsrcNO2_3D(i,j,k) = QsrcNO2(is,k)
!            enddo
!        endif
!      enddo


!             write(*,*) 'Qsrc_1 = ',Qsrc(1,:)
!            write(*,*) 'QsrcNH4_1 = ',QsrcNH4(1,:)

!        write(*,*) 'Qsrc1 = ',Qsrc(1,:)
!        write(*,*) 'QsrcNO3_2 = ',QpsrcNO3_3D(Isrc(2),jsrc(2),:)
!        write(*,*) 'QsrcNO3_4 = ',QpsrcNO3_3D(Isrc(4),jsrc(4),:)
!   stop
!  Unable to set-up Qbar and Tsrc0:
!  Complain about the error and signal to quit.
      else
        if (cff1.lt.0. .or. cff2.lt.0.) then
            write(*,'(/1x,3A/3(1x,A,F16.10)/)')      '### WARNING: ',
     &    'set_psource_tile :: Current model time is outside bounds ',
     &    'of ''psrc_time''.', 'psrc_tstart =', psrc_time(it1)*sec2day,
     &                         'tdays =',       tdays,
     &                         'psrc_tend =',   psrc_time(it2)*sec2day
          may_day_flag=2
        endif
        ierr=ierr+1
      endif

  1   format(1x,'WARNING: get_psrcs :: Cannot find variable ''',
     &                          A, ''' in file ''', A, '''.')

  2   format(/1x,'### ERROR: get_psrcs :: Cannot read variable ''',
     &                                A, ''' for rec =', I4/12x,A/)


      return

  99  may_day_flag=2

      return
      end

#else
      subroutine get_psrcs_empty
      end
#endif /* PSOURCE && !ANA_PSOURCE */
