#include "cppdefs.h"
!#if defined PSOURCE && !defined ANA_PSOURCE
#if defined PSOURCE
      subroutine get_psrcs (ierr)
 
! Read in point sources of flow and tracer at the appropriate
! time from psource NetCDF files.  Written by Eileen Deng-Idica
! (05-03-2007) & modified by Yusuke Uchiyama (06-20-2011)
 
      implicit none
# include "param.h"
# include "forces.h"
# include "sources.h"
# include "scalars.h"
# include "netcdf.inc"
# include "ncvars.h"
# ifdef MPI
#  include "hidden_mpi_vars.h"
# endif

      real cff, csum
      integer ierr, lstr,lvar,lenstr,ierr_count,ifrc, lfrc
      integer is, itrc, ncid, varid, i,k
#ifdef MPI
     &                        , i1,i2,j1,j2
#endif
      integer, parameter :: pdims=2
      integer            :: start(pdims), count(pdims)

      real,    dimension(:,:), allocatable :: D
      integer, dimension(:,:), allocatable :: C
      character(len=7)  snsrc , sk
      character(len=64) fmt
      character*(max_name_size) frcname

# if defined BULK_FLUX
        ifrc=1                 ! since tidal forcing data is read
 100    ncid=ncidbulk(ifrc)     ! only once and does not to be re-read
        frcname=blkfile(ifrc)  ! later, there is no need to memorize
        lfrc=lenstr(frcname)   ! netcdf ID for tidal file (unlike for
        ierr=nf_noerr          ! all other forces).
        ierr_count=0
# else
        ifrc=1                 ! since tidal forcing data is read
 100    ncid=ncidfrc(ifrc)     ! only once and does not to be re-read
        frcname=frcfile(ifrc)  ! later, there is no need to memorize
        lfrc=lenstr(frcname)   ! netcdf ID for tidal file (unlike for
        ierr=nf_noerr          ! all other forces).
        ierr_count=0
# endif

        if (ncid.eq.-1) then
# if defined BULK_FLUX
          ierr=nf_open (frcname(1:lfrc), nf_nowrite, ncid)
          if (ierr .eq. nf_noerr) then
            ncidbulk(ifrc)=ncid
          else
            write(*,'(/1x,4A/)') '### ERROR in get_psrc: Can not ',
     &                'open netCDF file ''', frcname(1:lfrc), '''.',
     &                                            nf_strerror(ierr)
          endif
# else
          ierr=nf_open (frcname(1:lfrc), nf_nowrite, ncid)
          if (ierr .eq. nf_noerr) then
            ncidfrc(ifrc)=ncid
          else
            write(*,'(/1x,4A/)') '### ERROR in get_psrc: Can not ',
     &                'open netCDF file ''', frcname(1:lfrc), '''.',
     &                                            nf_strerror(ierr)
          endif
# endif
        endif


! Nsrc: number of point sources

        ierr=nf_inq_dimid (ncid, 'Nsrc', varid)
      if (ierr .eq. nf_noerr) then
          ierr=nf_inq_dimlen (ncid, varid, Nsrc)
            write(*,*) 'Nsrc', frcname(1:lstr)
!            goto 99                                      !--> ERROR
            if (Nsrc.gt.Msrc) then
              write(*,*) 'FATAL ERROR: get_psrcs :: Found Nsrc > Msrc',
     &         '''',' Msrc = ',Msrc,' Nsrc = ',Nsrc
!              goto 99                                    !--> ERROR
            endif
      endif

! time
        ierr=nf_inq_varid (ncid, 'psrc_time', varid)
!        write(*,*) 'psrc_time is = ', psrc_time   ! KESSOURI
        if (ierr .eq. nf_noerr) then
          call set_cycle (ncid, psrc_tid, ntpsrc, psrc_cycle,
     &                    psrc_ncycle, psrc_rec, ierr)
          itpsrc=1
          psrc_time(1)=-1.E+20
          psrc_time(2)=-1.E+20
        else
          write(*,*) 'psrc_time', frcname(1:lstr)
!          goto 99                                        !--> ERROR
        endif

CR        write(*,*) 'ntpsrc=',ntpsrc
CR        write(*,*) 'psrc_rec=',psrc_rec
CR        write(*,*) 'psrc_cycle=',psrc_cycle
CR        write(*,*) 'psrc_ncycle=',psrc_ncycle
CR        write(*,*) 'psrc_time=',psrc_time(1)

!  99  may_day_flag=2
      return
      end
 
 
      subroutine set_psource_tile (istr,iend,jstr,jend, ierr)
 
! Set-up psource data for current tile.
 
      implicit none
      integer istr,iend,jstr,jend, ierr, it1,it2
      integer is, itrc, k
      real cff, cff1,cff2
# include "param.h"
# include "forces.h"
# include "sources.h"
# include "scalars.h"
# include "ocean3d.h"
 
c**
c         integer itpsrc
c         real psrc_time(2), psrc_cycle
c         real Tsrc0(Msrc,NT), TsrcG(Msrc,2,NT)
 
c**
 
# include "compute_extended_bounds.h"
 
      it1=3-itpsrc
      it2=itpsrc
      cff=time+0.5*dt
      cff1=psrc_time(it2)-cff
      cff2=cff-psrc_time(it1)
 
!  Load time invariant data.
 
      if (psrc_cycle.lt.0.) then
        if (FIRST_TIME_STEP) then
          do is=1,Nsrc
            Qbar(is)=QbarG(is,itpsrc)
            do k=1,N
              Qsrc(is,k)=Qbar(is)*Qshape(is,k)
            enddo
            do itrc=1,NT
 
                Tsrc0(is,itrc)=TsrcG(is,itpsrc,itrc)
                do k=1,N
                  Tsrc(is,k,itrc)=Tsrc0(is,itrc)
                enddo
 
            enddo
          enddo
        endif
 
!  Time-interpolate Qbar and Tsrc0 from point data.
!  Check that for the next time step [when time=time+dt] time+dt
!  is still between psrc_time(it1) and psrc_time(it2); and if not,
!  set synchro_flag top signal that the new forcing data should be
!  read from the netCDF input file (master thread only).
 
      elseif (cff1.ge.0. .and. cff2.ge.0.) then
        if (cff1.lt.dt) synchro_flag=.true.
 
        cff=1./(cff1+cff2)
        cff1=cff1*cff
        cff2=cff2*cff
        do is=1,Nsrc
          Qbar(is)=cff1*QbarG(is,it1)+cff2*QbarG(is,it2)
c*          if (QbarG(is,it1) .eq. 0. .or. QbarG(is,it2) .eq.0) then
c*              Qbar(is) = 0.  ! makes sure set_psource doesn't
c*          endif              ! interpolate more flow than you want
          do k=1,N
            Qsrc(is,k)=Qbar(is)*Qshape(is,k)
          enddo
          do itrc=1,NT
 
              Tsrc0(is,itrc)=cff1*TsrcG(is,it1,itrc)
     &                     + cff2*TsrcG(is,it2,itrc)
              do k=1,N
                 Tsrc(is,k,itrc)=Tsrc0(is,itrc)
              enddo
 
          enddo
        enddo
 
!  Unable to set-up Qbar and Tsrc0:
!  Complain about the error and signal to quit.
 
      else
        if (ZEROTH_TILE) then
            write(*,'(/1x,3A/3(1x,A,F16.10)/)')      '### WARNING: ',
     &    'set_psource_tile :: Current model time is outside bounds ',
     &    'of ''psrc_time''.', 'psrc_tstart =', psrc_time(it1)*sec2day,
     &                         'tdays =',       tdays,
     &                         'psrc_tend =',   psrc_time(it2)*sec2day
          may_day_flag=2
        endif
        ierr=ierr+1
      endif
      return
      end
#else
      subroutine get_psource_empty
      end
#endif /* PSOURCE && !ANA_PSOURCE */
