! Dimensions of Physical Grid and array dimensions:
!----------- -- -------- ---- --- ----- -----------
! LLm   Number of the internal points of the PHYSICAL grid in XI-
! MMm   and ETA-directions, excluding physical side boundary points,
!       peroodic ghost points, and MPI-margins (if any).
!
! Lm    Number of the internal points [see above] of array covering
! Mm    a single MPI-subdomain.  These are identical to LLm, MMm if
!       there is no MPI-partitioning.

      integer, parameter ::
#if defined L2SCB
     &               LLm=600, MMm=1410, N=60
#endif


!
! MPI related variables
! === ======= =========
!
      integer Lmmpi,Mmmpi,iminmpi,imaxmpi,jminmpi,jmaxmpi
      common /comm_setup_mpi/ Lmmpi,Mmmpi,
     &                    iminmpi,imaxmpi,jminmpi,jmaxmpi


! Domain subdivision parameters:
!------- ----------- -----------
! NNODES             total number of MPI processes (nodes);
! NP_XI,  NP_ETA     number of MPI subdomains in XI-, ETA-directions;
! NSUB_X, NSUB_E     number of shared memory subdomains (tiles) in
!                                             XI- and ETA-directions;
      integer, parameter ::
#ifdef MPI
     &      NP_XI=12, NP_ETA=30, NSUB_X=1, NSUB_E=1
#else
c     &      NSUB_X=4, NSUB_E=40  ! PAC44
c     &      NSUB_X=8, NSUB_E=80   ! PAC22
c     &      NSUB_X=1, NSUB_E=8
     &      NSUB_X=1, NSUB_E=1
c     &      NSUB_X=4, NSUB_E=26  ! <-- 384x128 soliton
c     &      NSUB_X=4, NSUB_E=52
c     &      NSUB_X=4, NSUB_E=24  ! <-- iswake 768x192
c     &      NSUB_X=8, NSUB_E=48
#endif

! Array dimensions and bounds of the used portions of sub-arrays

#ifdef MPI
      integer, parameter :: NNODES=NP_XI*NP_ETA,
     &    Lm=(LLm+NP_XI-1)/NP_XI, Mm=(MMm+NP_ETA-1)/NP_ETA

      integer ocean_grid_comm, mynode,  iSW_corn, jSW_corn,
     &                         iwest, ieast, jsouth, jnorth
# ifndef EW_PERIODIC
      logical west_exchng,  east_exchng
# endif
# ifndef NS_PERIODIC
      logical south_exchng, north_exchng
# endif
      common /mpi_comm_vars/  ocean_grid_comm, mynode,
     &     iSW_corn, jSW_corn, iwest, ieast, jsouth, jnorth
# ifndef EW_PERIODIC
     &                , west_exchng,  east_exchng
# endif
# ifndef NS_PERIODIC
     &                , south_exchng, north_exchng
# endif
#else
      integer, parameter :: Lm=LLm, Mm=MMm
#endif

! Derived dimension parameters, number of tracers and tracer
! identification indices:
      integer, parameter :: padd_X=(Lm+2)/2-(Lm+1)/2,
     &                      padd_E=(Mm+2)/2-(Mm+1)/2
#ifdef SOLVE3D
     &       , itemp=1
# ifdef SALINITY
     &       , isalt=2
     &       , ntrc_salt=1
#  ifdef BIOLOGY
     &       , iNO3_=3, iNH4_=4, iDet_=5, iPhyt=6, iZoo_=7
#  endif
# else
             , ntrc_salt=0
#  ifdef BIOLOGY
     &       , iNO3_=2, iNH4_=3, iDet_=4, iPhyt=5, iZoo_=6
#  endif
# endif
# ifdef PASSIVE_TRACER
#  ifdef TRACER_BE7
      integer, parameter :: ntrc_pas=1
#  else /* TRACER_BE7 */
      integer, parameter :: ntrc_pas=6
#  endif /* TRACER_BE7 */
      integer itpas(ntrc_pas)
      common /pass_tracer/ itpas
# else
      integer, parameter :: ntrc_pas=0
# endif

# ifdef BIOLOGY_BEC
      integer, parameter :: itrc_bio=itemp+ntrc_salt+ntrc_pas+1
      integer, parameter :: iPO4=itrc_bio,iNO3=iPO4+1, iSIO3=iPO4+2, 
     &     iNH4=iPO4+3,
     &     iFE=iPO4+4, iO2=iPO4+5, iDIC=iPO4+6,
     &     iALK=iPO4+7, iDOC=iPO4+8, iSPC=iPO4+9,
     &     iSPCHL=iPO4+10, iSPCACO3=iPO4+11, iDIATC=iPO4+12,
     &     iDIATCHL=iPO4+13, iZOOC=iPO4+14, iSPFE=iPO4+15,
     &     iDIATSI=iPO4+16, iDIATFE=iPO4+17, iDIAZC=iPO4+18,
     &     iDIAZCHL=iPO4+19, iDIAZFE=iPO4+20, iDON=iPO4+21,
     &     iDOFE=iPO4+22, iDOP=iPO4+23
#  ifdef DMS_CYCLE
      integer, parameter :: iDMSP=iDOP+1,iDMS=iDMSP+1
#  endif /* DMS_CYCLE */
#  ifdef NCYCLE_ANOXIC
      integer, parameter :: iNO2 = iDOP + 1
#   ifdef DMS_CYCLE
     &     + 2
#   endif /* DMS_CYCLE */
      integer, parameter :: iN2O = iNO2 + 1, iN2 = iN2O + 1
#  endif /* NCYCLE_ANOXIC */
#  ifdef N15_CYCLE
      integer, parameter ::iNO3_15N = iDOP + 1
#   ifdef DMS_CYCLE
     &     + 2
#   endif /* DMS_CYCLE */
#   ifdef NCYCLE_ANOXIC
     &     + 3
#   endif /* NCYCLE_ANOXIC */
      integer, parameter :: iNH4_15N = iNO3_15N + 1
      integer, parameter :: iSp_15N = iNH4_15N + 1
      integer, parameter :: iDiat_15N = iSp_15N + 1
      integer, parameter :: iZoo_15N = iDiat_15N + 1
      integer, parameter :: iDiaz_15N = iZoo_15N + 1
      integer, parameter :: iDON_15N = iDiaz_15N + 1
#   ifdef NCYCLE_ANOXIC
      integer, parameter :: iNO2_15N = iDON_15N + 1
      integer, parameter :: iN2O_15N = iNO2_15N + 1
      integer, parameter :: iN2_15N = iN2O_15N + 1
#   endif /* NCYCLE_ANOXIC */
#  endif /* N15_CYCLE */

      integer, parameter :: ntrc_bio = 24
#  ifdef DMS_CYCLE
     &     + 2
#  endif /* DMS_CYCLE */
#  ifdef NCYCLE_ANOXIC
     &     + 3
#  endif /* NCYCLE_ANOXIC */
#  ifdef N15_CYCLE
     &     + 7
#   ifdef NCYCLE_ANOXIC
     &     + 3
#   endif /* NCYCLE_ANOXIC */
#  endif /* N15_CYCLE */
#  ifdef SEDIMENT_BIOLOGY
      integer, parameter :: NT_sed = 3      
#   ifdef N15_CYCLE
     &     + 1
#   endif 
#  endif /* SEDIMENT_BIOLOGY */
# else
      integer, parameter :: ntrc_bio = 0
# endif /* BIOLOGY_BEC */
      integer, parameter :: NT=itemp+ntrc_salt+ntrc_pas+ntrc_bio
#endif /*SOLVE3D */

#ifdef PSOURCE
      integer, parameter :: Msrc=182   ! Number of point sources
      integer, parameter :: Stepsrc=5858   ! Number of time steps of the point sources
#endif
#ifdef STATIONS
      integer, parameter :: NS=5      ! Number of output stations
#endif
#ifdef FLOATS
      integer, parameter :: Mfloats=32000 ! Maximum number of floats
#endif
