! Compute r.h.s. terms for 3D momentum equations (except pressure
! gradient). Note that arrays "ru","rv" are expected to be initialized
! at entry into this routine by a pressure gradient scheme.

#include "cppdefs.h"
#ifdef SOLVE3D
# define UPSTREAM_UV

      subroutine rhs3d (tile)
      implicit none
      integer tile
# include "param.h"
# include "private_scratch.h"
# include "compute_tile_bounds.h"
      call rhs3d_tile (istr,iend,jstr,jend,  A3d(1,1), A3d(1,2),
     &                       A2d(1,1),A2d(1,2),A2d(1,3),A2d(1,4),
     &                       A2d(1,1),A2d(1,2),A2d(1,3),A2d(1,4),
     &                                        A2d(1,5),A2d(1,6))
      return
      end

      subroutine rhs3d_tile (istr,iend,jstr,jend, ru,rv, FC,WC,CF,DC,
     &                                    UFx,UFe,VFx,VFe, wrk1,wrk2)
      implicit none
# include "param.h"
      integer istr,iend,jstr,jend, imin,imax,jmin,jmax, i,j,k
      real, dimension(PRIVATE_2D_SCRATCH_ARRAY,N) :: ru,rv
      real, dimension(PRIVATE_1D_SCRATCH_ARRAY,0:N) :: FC,WC,CF,DC
      real, dimension(PRIVATE_2D_SCRATCH_ARRAY) :: UFx,UFe,VFx,VFe,
     &                                                   wrk1,wrk2
      real dtau, cff, cff1
      real, parameter ::  delta=0.125 !!!! delta=0.1666666666666667
# ifdef UPSTREAM_UV
     &                  , gamma=0.25  !!!! gamma=0.3333333333333333
# endif
# include "grid.h"
# include "ocean3d.h"
# include "coupling.h"
# include "forces.h"
# include "scalars.h"
# ifdef IMPLICIT_BOTTOM_DRAG
#  include "mixing.h"
# endif

# include "compute_auxiliary_bounds.h"

      if (CORR_STAGE) then
        dtau=dt
      elseif (FIRST_TIME_STEP) then
        dtau=0.5*dt
      else
        dtau=(1.-1./6.)*dt
      endif

#undef COUPLED_STAGE_ONLY
#define COUPLED_STAGE_ONLY .true.

       do k=1,N
#include "compute_horiz_rhs_uv_terms.h"
       enddo

! Compute dynamic bottom drag coefficient.  Note that "rd" must be
! aliased to a horizontal array beyond #4 to avoid overlap with one
! of the vertical array used below.

# define rd wrk1
#include "compute_rd_bott_drag.h"

      do j=jstr,jend

#include "compute_vert_rhs_uv_terms.h"

! Start computation of the forcing terms for the 2D (barotropic mode)
! momentum equations: vertically integrate the just computed r.h.s
! "ru" and "rv". Also, if so prescribed, add in the difference between
! surface (wind) and bottom (drag) stresses. The computation of the
! 3D-->2D forcing terms will be finalized in "step2d" during the first
! barotropic time step, by subtracting the barotropic r.h.s. "rubar"
! and "rvbar" from the vertical integrals "rufrc" and "rvfrc" computed
! here.

        if (COUPLED_STAGE_ONLY) then
# ifdef IMPLICIT_BOTTOM_DRAG



          do i=istrU,iend
            DC(i,0)=dtau*0.25*(pm(i,j)+pm(i-1,j))*(pn(i,j)+pn(i-1,j))

            FC(i,N-1)= 2.*dtau*(Akv(i,j,N-1)+Akv(i-1,j,N-1))
     &                          /(  Hz(i,j,N  )+Hz(i-1,j,N  )
     &                             +Hz(i,j,N-1)+Hz(i-1,j,N-1))

            WC(i,N-1)= DC(i,0)*0.5*(Wi(i,j,N-1)+Wi(i-1,j,N-1))

            cff=1./( 0.5*(Hz(i,j,N)+Hz(i-1,j,N))
     &                     +FC(i,N-1)-min(WC(i,N-1),0.) )
            CF(i,N-1)=cff*( FC(i,N-1)+max(WC(i,N-1),0.) )

            if (CORR_STAGE) then
              DC(i,N)=u(i,j,N,nnew)
            else
              DC(i,N)=u(i,j,N,nstp)*0.5*(Hz(i,j,N)+Hz(i-1,j,N))
            endif

            DC(i,N)=cff*(DC(i,N) +DC(i,0)*ru(i,j,N) +dtau*sustr(i,j))
          enddo
          do k=N-1,2,-1      !--> forward elimination
            do i=istrU,iend
              FC(i,k-1)= 2.*dtau*(Akv(i,j,k-1)+Akv(i-1,j,k-1))
     &                            /(  Hz(i,j,k  )+Hz(i-1,j,k  )
     &                              +Hz(i,j,k-1)+Hz(i-1,j,k-1))

              WC(i,k-1)= DC(i,0)*0.5*(Wi(i,j,k-1)+Wi(i-1,j,k-1))

              cff=1./( 0.5*(Hz(i,j,k)+Hz(i-1,j,k))
     &                              +FC(i,k-1)-min(WC(i,k-1),0.)
     &                                +FC(i,k)+max(WC(i,k),0.)
     &                       -CF(i,k)*(FC(i,k)-min(WC(i,k),0.))
     &                                                         )

              CF(i,k-1)=cff*( FC(i,k-1)+max(WC(i,k-1),0.) )

              if (CORR_STAGE) then
                DC(i,k)=u(i,j,k,nnew)
              else
                DC(i,k)=u(i,j,k,nstp)*0.5*(Hz(i,j,k)+Hz(i-1,j,k))
              endif

              DC(i,k)=cff*( DC(i,k) +DC(i,0)*ru(i,j,k)
     &                    +DC(i,k+1)*(FC(i,k)-min(WC(i,k),0.)) )
            enddo
          enddo
          do i=istrU,iend
            if (CORR_STAGE) then
              DC(i,1)=u(i,j,1,nnew)
            else
              DC(i,1)=u(i,j,1,nstp)*0.5*(Hz(i,j,1)+Hz(i-1,j,1))
            endif

            DC(i,1)=( DC(i,1) +DC(i,0)*ru(i,j,1)
     &                     +DC(i,2)*(FC(i,1)-min(WC(i,1),0.))
     &                 )/( 0.5*(Hz(i,j,1)+Hz(i-1,j,1))
#  ifdef IMPLCT_NO_SLIP_BTTM_BC
     &                            +0.5*dtau*(rd(i,j)+rd(i-1,j))
#  endif
     &                                  +FC(i,1)+max(WC(i,1),0.)
     &                      -CF(i,1)*(FC(i,1)-min(WC(i,1),0.)) )
          enddo


          cff=1./dtau
          do k=2,N,+1          !--> backsubstitution
            do i=istrU,iend
              DC(i,k)=DC(i,k) +CF(i,k-1)*DC(i,k-1)
!>
              if (WC(i,k-1).gt.0.) then
                cff1=DC(i,k-1)
              else
                cff1=DC(i,k)
              endif
              FC(i,k-1)=cff*( FC(i,k-1)*(DC(i,k)-DC(i,k-1))
     &                                     -cff1*WC(i,k-1) )
            enddo
          enddo       !--> now FC is implicit adv + visc. flux

          do i=istrU,iend
            DC(i,0)=dm_u(i,j)*dn_u(i,j)
            ru(i,j,N)=ru(i,j,N) +DC(i,0)*(sustr(i,j)-FC(i,N-1))
            ru(i,j,1)=ru(i,j,1) +DC(i,0)*( FC(i,1)
     &                        -0.5*(rd(i-1,j)+rd(i,j))*DC(i,1) )

            rufrc(i,j)=ru(i,j,N)+ru(i,j,1)
          enddo
          do k=N-1,2,-1
            do i=istrU,iend
              ru(i,j,k)=ru(i,j,k) +DC(i,0)*(FC(i,k)-FC(i,k-1))

              rufrc(i,j)=rufrc(i,j)+ru(i,j,k)
            enddo
          enddo



          if (j.ge.jstrV) then
            do i=istr,iend
              DC(i,0)=dtau*0.25*(pm(i,j)+pm(i,j-1))*(pn(i,j)+pn(i,j-1))

              FC(i,N-1)= 2.*dtau*(Akv(i,j,N-1)+Akv(i,j-1,N-1))
     &                            /(  Hz(i,j,N  )+Hz(i,j-1,N  )
     &                               +Hz(i,j,N-1)+Hz(i,j-1,N-1))

              WC(i,N-1)= DC(i,0)*0.5*(Wi(i,j,N-1)+Wi(i,j-1,N-1))

              cff=1./( 0.5*(Hz(i,j,N)+Hz(i,j-1,N))
     &                       +FC(i,N-1)-min(WC(i,N-1),0.) )

              CF(i,N-1)=cff*( FC(i,N-1)+max(WC(i,N-1),0.) )

              if (CORR_STAGE) then
                DC(i,N)=v(i,j,N,nnew)
              else
                DC(i,N)=v(i,j,N,nstp)*0.5*(Hz(i,j,N)+Hz(i,j-1,N))
              endif

              DC(i,N)=cff*(DC(i,N) +DC(i,0)*rv(i,j,N) +dtau*svstr(i,j))
            enddo
            do k=N-1,2,-1      !--> forward elimination
              do i=istr,iend
                FC(i,k-1)= 2.*dtau*(Akv(i,j,k-1)+Akv(i,j-1,k-1))
     &                              /(  Hz(i,j,k  )+Hz(i,j-1,k  )
     &                                +Hz(i,j,k-1)+Hz(i,j-1,k-1))

                WC(i,k-1)= DC(i,0)*0.5*(Wi(i,j,k-1)+Wi(i,j-1,k-1))

                cff=1./( 0.5*(Hz(i,j,k)+Hz(i,j-1,k))
     &                            +FC(i,k-1)-min(WC(i,k-1),0.)
     &                                  +FC(i,k)+max(WC(i,k),0.)
     &                         -CF(i,k)*(FC(i,k)-min(WC(i,k),0.))
     &                                                          )
                CF(i,k-1)=cff*( FC(i,k-1)+max(WC(i,k-1),0.) )

                if (CORR_STAGE) then
                  DC(i,k)=v(i,j,k,nnew)
                else
                  DC(i,k)=v(i,j,k,nstp)*0.5*(Hz(i,j,k)+Hz(i,j-1,k))
                endif

                DC(i,k)=cff*( DC(i,k) +DC(i,0)*rv(i,j,k)
     &                    +DC(i,k+1)*(FC(i,k)-min(WC(i,k),0.)) )
              enddo
            enddo
            do i=istr,iend
              if (CORR_STAGE) then
                DC(i,1)=v(i,j,1,nnew)
              else
                DC(i,1)=v(i,j,1,nstp)*0.5*(Hz(i,j,1)+Hz(i,j-1,1))
              endif

              DC(i,1)=( DC(i,1) +DC(i,0)*rv(i,j,1)
     &                       +DC(i,2)*(FC(i,1)-min(WC(i,1),0.))
     &                     )/( 0.5*(Hz(i,j,1)+Hz(i,j-1,1))
#  ifdef IMPLCT_NO_SLIP_BTTM_BC
     &                            +0.5*dtau*(rd(i,j)+rd(i,j-1))
#  endif
     &                                  +FC(i,1)+max(WC(i,1),0.)
     &                      -CF(i,1)*(FC(i,1)-min(WC(i,1),0.)) )
            enddo
            cff=1./dtau
            do k=2,N,+1          !--> backsubstitution
              do i=istr,iend
                DC(i,k)=DC(i,k) +CF(i,k-1)*DC(i,k-1)
!>
                FC(i,k-1)=cff*FC(i,k-1)*(DC(i,k)-DC(i,k-1))
              enddo
            enddo        !--> now FC is visc. flux

            do i=istr,iend
              DC(i,0)=dm_v(i,j)*dn_v(i,j)
              rv(i,j,N)=rv(i,j,N) +DC(i,0)*(svstr(i,j)-FC(i,N-1))
              rv(i,j,1)=rv(i,j,1) +DC(i,0)*( FC(i,1)
     &                          -0.5*(rd(i,j-1)+rd(i,j))*DC(i,1) )

              rvfrc(i,j)=rv(i,j,N)+rv(i,j,1)
            enddo
            do k=N-1,2,-1
              do i=istr,iend
                rv(i,j,k)=rv(i,j,k) +DC(i,0)*(FC(i,k)-FC(i,k-1))

                rvfrc(i,j)=rvfrc(i,j)+rv(i,j,k)
              enddo
            enddo
          endif


# else
                                               ! Note that in the case
          do i=istrU,iend                      ! of NO SLIP BOTTOM only
            cff=dm_u(i,j)*dn_u(i,j)            ! rufrc receives bottom
            ru(i,j,N)=ru(i,j,N) +cff*sustr(i,j)
#  ifdef IMPLCT_NO_SLIP_BTTM_BC
            rufrc(i,j)=ru(i,j,N) +ru(i,j,1) -cff*u(i,j,1,nstp) !<-- nrhs
     &                                *0.5*(rd(i-1,j)+rd(i,j))
#  else
            ru(i,j,1)=            ru(i,j,1) -cff*u(i,j,1,nstp) !<-- nrhs
     &                                *0.5*(rd(i-1,j)+rd(i,j))

            rufrc(i,j)=ru(i,j,N)+ru(i,j,1)
#  endif
          enddo                                ! stress; the 3D part
          do k=N-1,2,-1                        ! gets implicit bottom
            do i=istrU,iend                    ! drag as the result of
              rufrc(i,j)=rufrc(i,j)+ru(i,j,k)  ! vertical tri-diagonal
            enddo                              ! viscous solver inside
          enddo                                ! step3d_uv2.

          if (j.ge.jstrV) then
            do i=istr,iend
              cff=dm_v(i,j)*dn_v(i,j)
              rv(i,j,N)=rv(i,j,N) +cff*svstr(i,j)
#  ifdef IMPLCT_NO_SLIP_BTTM_BC
              rvfrc(i,j)=rv(i,j,N)+rv(i,j,1) -cff*v(i,j,1,nstp)
     &                                 *0.5*(rd(i,j-1)+rd(i,j))
#  else
              rv(i,j,1)=           rv(i,j,1) -cff*v(i,j,1,nstp)
     &                                 *0.5*(rd(i,j-1)+rd(i,j))

              rvfrc(i,j)=rv(i,j,N)+rv(i,j,1)
#  endif
            enddo
            do k=N-1,2,-1
              do i=istr,iend
                rvfrc(i,j)=rvfrc(i,j)+rv(i,j,k)
              enddo
            enddo
          endif
# endif
# undef rd
        endif   ! <-- COUPLED_STAGE_ONLY
      enddo      !<-- j
      return
      end

      subroutine check_rhs3d_switches (ierr)

! This routine keeps track of the status of local CPP-settings in
! "rhs3d35S.F". This code must be placed here rather than a separate
! file in order to be exposed to the relevant CPP-settings.  It does
! not affect any model results, other than "CPPS" signature saved as
! a global attribute in output netCDF files.

      implicit none
      integer ierr, is,ie, lenstr
# include "param.h"
# include "strings.h"
      ie=lenstr(cpps)
      is=ie+2
      ie=is+11
      if (ie.gt.max_opt_size) goto 99
      cpps(is:ie)='<rhs3d35S.F>'
# ifdef UPSTREAM_UV
      is=ie+2
      ie=is+10
      if (ie.gt.max_opt_size) goto 99
      cpps(is:ie)='UPSTREAM_UV'
# endif
      return
  99  mpi_master_only write(*,'(/1x,2A/12x,A/)')      '### ERROR: ',
     &  'Insufficient length of string "cpps" in file "strings.h".',
     &        'Increase parameter "max_opt_size" it and recompile.'
      ierr=ierr+1
      return
      end
#else
      subroutine rhs3d_empty
      end
#endif /* SOLVE3D */
