!
! $Id: set_avg.F,v 1.8 2005/09/23 15:56:31 pmarches Exp $
!
#include "cppdefs.h"
#if defined AVERAGES && defined OUTPUT_AVG_PHYS_BIO
      subroutine set_bio_avg (tile)
      implicit none
      integer tile
# include "param.h"
# ifdef SOLVE3D
#  include "work.h"
#  include "ncvars.h"
# endif
# include "compute_tile_bounds.h"

      call set_bio_avg_tile (Istr,Iend,Jstr,Jend)

      return
      end

      subroutine set_bio_avg_tile (Istr,Iend,Jstr,Jend)
!
! Compute time-averaged fields within a tile.
! ------- ------------- ------ ------ - -----
! Because of syncronization issues, the delayed mode averaging
! procedure is used. This procedure implies that all fields to be
! averaged are sampled during the next time step, rather than at
! the end of the time step when they were computed.
!
! Though this algorithm results in somewhat ackward controlling
! logic it has the advantage that all fields to be sampled
! correspond to exactly the same time, which is time step "n".
! Particularly, this is done this way because vertical velocity
! corresponding to the newly computed horizontal velocities
! becomes available only during the following time step.
! The same applies to the density field.
!
! The algorithm consists of three logical blocks: (1) initialization
! of the averages arrays: when mod(ilc-1,n_bio_avg).eq.1 the target arrays
! are set to the first contribution; (2) accumulation of averaged
! data, when mod(ilc-1,n_bio_avg).gt.1; and (3) adding the last
! contribution and scaling.
!
      implicit none
# include "param.h"
      integer Istr,Iend,Jstr,Jend, i,j, ilc
      real cff
# ifdef SOLVE3D
      integer itrc,k
#  include "work.h"
# endif
# include "scalars.h"
# include "ncvars.h"
# include "grid.h"
# include "ocean2d.h"
# include "ocean3d.h"
# include "averages.h"
# include "mixing.h"
# include "forces.h"
# ifdef SEDIMENT_BIOLOGY
#  include "bgcflux.h"
# endif
# if defined BIOLOGY_BEC
#  include "ecosys_bec.h"
#  include "bec_avg.h"
# endif
!
#include "compute_auxiliary_bounds.h"
!
      ilc=1+iic-ntstart   ! number of time step since restart
!
      if (ilc.gt.nts_bio_avg) then 
        if (mod(ilc-1,n_bio_avg).eq.1) then
          if (wrtavg(indxZ)) then
            do j=JstrR,JendR
              do i=IstrR,IendR
                zeta_bio_avg(i,j)=zeta(i,j,fast_indx_out)
              enddo
            enddo
          endif
#ifdef BIOLOGY
          if (wrtavg(indxHel)) then
            do j=JstrR,JendR
              do i=IstrR,IendR
                hel_avg(i,j)=hel(i,j)
              enddo
            enddo
          endif
# ifdef BIO_NPZD
          if (wrtavg(indxChC)) then
            do k=1,N
              do j=JstrR,JendR
                do i=IstrR,IendR
                  theta_avg(i,j,k)=theta(i,j,k)
                enddo
              enddo
            enddo
          endif
#  ifdef OXYGEN
          if (wrtavg(indxU10)) then
             do j=JstrR,JendR
                do i=IstrR,IendR
                   u10_avg(i,j)=u10(i,j)
                enddo
             enddo
          endif
          if (wrtavg(indxKvO2)) then
             do j=JstrR,JendR
                do i=IstrR,IendR
                   Kv_O2_avg(i,j)=Kv_O2(i,j)
                enddo
             enddo
          endif
          if (wrtavg(indxO2sat)) then
             do j=JstrR,JendR
                do i=IstrR,IendR
                   O2satu_avg(i,j)=O2satu(i,j)
                enddo
             enddo
          endif
#   endif /* OXYGEN */
# elif defined BIO_N2P2Z2D2 && defined VAR_CHL_C
          if (wrtavg(indxChC1)) then
            do k=1,N
              do j=JstrR,JendR
                do i=IstrR,IendR
                  theta1_avg(i,j,k)=theta1(i,j,k)
                enddo
              enddo
            enddo
          endif

          if (wrtavg(indxChC2)) then
            do k=1,N
              do j=JstrR,JendR
                do i=IstrR,IendR
                  theta2_avg(i,j,k)=theta2(i,j,k)
                enddo
              enddo
            enddo
          endif
# endif
#endif /* BIOLOGY */
# ifdef SOLVE3D
          do itrc=2+ntrc_salt,NT
            if (wrtavg(indxT+itrc-1)) then
              do k=1,N
                do j=JstrR,JendR
                  do i=IstrR,IendR
                    t_avg(i,j,k,itrc)=t(i,j,k,nstp,itrc)
                  enddo
                enddo
              enddo
            endif
          enddo
#  ifdef SEDIMENT_BIOLOGY
          do itrc=1,NT_sed
             if (wrtavg(indxSedOrgN+itrc-1)) then
                do j=JstrR,JendR
                   do i=IstrR,IendR
                      t_sed_avg(i,j,itrc)=t_sed(i,j,itrc)
                   enddo
                enddo
             endif
          enddo
#  endif /* SEDIMENT_BIOLOGY */
# endif /* SOLVE3D */
        elseif (mod(ilc-1,n_bio_avg).gt.1) then
          if (wrtavg(indxZ)) then
            do j=JstrR,JendR
              do i=IstrR,IendR
                zeta_bio_avg(i,j)=zeta_bio_avg(i,j)+
     &                zeta(i,j,fast_indx_out)
              enddo
            enddo
          endif
#ifdef BIOLOGY
          if (wrtavg(indxHel)) then
            do j=JstrR,JendR
              do i=IstrR,IendR
                hel_avg(i,j)=hel_avg(i,j)+hel(i,j)
              enddo
            enddo
          endif
# ifdef BIO_NPZD
          if (wrtavg(indxChC)) then
            do k=1,N
              do j=JstrR,JendR
                do i=IstrR,IendR
                  theta_avg(i,j,k)=theta_avg(i,j,k)+theta(i,j,k)
                enddo
              enddo
            enddo
          endif
#   ifdef OXYGEN
          if (wrtavg(indxU10)) then
             do j=JstrR,JendR
                do i=IstrR,IendR
                   u10_avg(i,j)=u10_avg(i,j)+u10(i,j)
                enddo
             enddo
          endif
          if (wrtavg(indxKvO2)) then
             do j=JstrR,JendR
                do i=IstrR,IendR
                   Kv_O2_avg(i,j)=Kv_O2_avg(i,j)+Kv_O2(i,j)
                enddo
             enddo
          endif
          if (wrtavg(indxO2sat)) then
             do j=JstrR,JendR
                do i=IstrR,IendR
                   O2satu_avg(i,j)=O2satu_avg(i,j)+O2satu(i,j)
                enddo
             enddo
          endif
#   endif /* OXYGEN */
# elif defined BIO_N2P2Z2D2 && defined VAR_CHL_C
          if (wrtavg(indxChC1)) then
            do k=1,N
              do j=JstrR,JendR
                do i=IstrR,IendR
                  theta1_avg(i,j,k)=theta1_avg(i,j,k)+theta1(i,j,k)
                enddo
              enddo
            enddo
          endif

          if (wrtavg(indxChC2)) then
            do k=1,N
              do j=JstrR,JendR
                do i=IstrR,IendR
                  theta2_avg(i,j,k)=theta2_avg(i,j,k)+theta2(i,j,k)
                enddo
              enddo
            enddo
          endif
# endif
#endif /* BIOLOGY */
# ifdef SOLVE3D
          do itrc=2+ntrc_salt,NT
            if (wrtavg(indxT+itrc-1)) then
              do k=1,N
                do j=JstrR,JendR
                  do i=IstrR,IendR
                    t_avg(i,j,k,itrc)=t_avg(i,j,k,itrc)
     &                              +t(i,j,k,nstp,itrc)
                  enddo
                enddo
              enddo
            endif
          enddo
#ifdef SEDIMENT_BIOLOGY
          do itrc=1,NT_sed
             if (wrtavg(indxSedOrgN+itrc-1)) then
                do j=JstrR,JendR
                   do i=IstrR,IendR
                      t_sed_avg(i,j,itrc)=t_sed_avg(i,j,itrc)
     &                     +t_sed(i,j,itrc)
                   enddo
                enddo
             endif
          enddo
#endif /* SEDIMENT_BIOLOGY */
# endif /* SOLVE3D */
        elseif (mod(ilc-1,n_bio_avg).eq.0) then
          cff=1./float(n_bio_avg)
          if (ZEROTH_TILE) time_bio_avg=time_bio_avg+float(n_bio_avg)*dt
          if (wrtavg(indxZ)) then
            do j=JstrR,JendR
              do i=IstrR,IendR
                zeta_bio_avg(i,j)=cff*( zeta_bio_avg(i,j)
     &                   +zeta(i,j,fast_indx_out))
              enddo
            enddo
          endif
#ifdef BIOLOGY
          if (wrtavg(indxHel)) then
            do j=JstrR,JendR
              do i=IstrR,IendR
                hel_avg(i,j)=cff*( hel_avg(i,j)
     &                            +hel(i,j)     )
              enddo
            enddo
          endif
# ifdef BIO_NPZD
          if (wrtavg(indxChC)) then
            do k=1,N
              do j=JstrR,JendR
                do i=IstrR,IendR
                  theta_avg(i,j,k)=cff*( theta_avg(i,j,k)
     &                                  +theta(i,j,k)     )
                enddo
              enddo
            enddo
          endif
#   ifdef OXYGEN
          if (wrtavg(indxU10)) then
             do j=JstrR,JendR
                do i=IstrR,IendR
                   u10_avg(i,j)=cff*( u10_avg(i,j)+u10(i,j) )
                enddo
             enddo
          endif
          if (wrtavg(indxKvO2)) then
             do j=JstrR,JendR
                do i=IstrR,IendR
                   Kv_O2_avg(i,j)=cff*( Kv_O2_avg(i,j)+Kv_O2(i,j) )
                enddo
             enddo
          endif
          if (wrtavg(indxO2sat)) then
             do j=JstrR,JendR
                do i=IstrR,IendR
                   O2satu_avg(i,j)=cff*( O2satu_avg(i,j)+O2satu(i,j) )
                enddo
             enddo
          endif
#   endif /* OXYGEN */
# elif defined BIO_N2P2Z2D2 && defined VAR_CHL_C
          if (wrtavg(indxChC1)) then
            do k=1,N
              do j=JstrR,JendR
                do i=IstrR,IendR
                  theta1_avg(i,j,k)=cff*( theta1_avg(i,j,k)
     &                                   +theta1(i,j,k)     )
                enddo
              enddo
            enddo
          endif

          if (wrtavg(indxChC2)) then
            do k=1,N
              do j=JstrR,JendR
                do i=IstrR,IendR
                  theta2_avg(i,j,k)=cff*( theta2_avg(i,j,k)
     &                                   +theta2(i,j,k)     )
                enddo
              enddo
            enddo
          endif
# endif
#endif /* BIOLOGY */
# ifdef SOLVE3D
          do itrc=2+ntrc_salt,NT
            if (wrtavg(indxT+itrc-1)) then
              do k=1,N
                do j=JstrR,JendR
                  do i=IstrR,IendR
                    t_avg(i,j,k,itrc)=cff*( t_avg(i,j,k,itrc)
     &                                    +t(i,j,k,nstp,itrc))
                  enddo
                enddo
              enddo
            endif
          enddo
#  ifdef SEDIMENT_BIOLOGY
          do itrc=1,NT_sed
             if (wrtavg(indxSedOrgN+itrc-1)) then
                do j=JstrR,JendR
                   do i=IstrR,IendR
                      t_sed_avg(i,j,itrc)=cff*( t_sed_avg(i,j,itrc)
     &                     +t_sed(i,j,itrc))
                   enddo
                enddo
             endif
          enddo
#  endif /* SEDIMENT_BIOLOGY */
#  if (defined BIOLOGY_NPZDOC || defined BIOLOGY_BEC)
          do j=jstrR,jendR
             do i=istrR,iendR
                pH_avg(i,j) = cff*( pH_avg(i,j) + pH_hist(i,j) )
                pCO2_avg(i,j) = cff*( pCO2_avg(i,j) + pCO2sw(i,j) )
                pCO2air_avg(i,j) = cff*(pCO2air_avg(i,j) + pCO2air(i,j))
             end do
          end do
          do j=jstrR,jendR
             do i=istrR,iendR
                PARinc_avg(i,j) = cff*( PARinc_avg(i,j) + PARinc(i,j) )
             end do
          end do
          do k=1,N
            do j=jstrR,jendR
               do i=istrR,iendR
                  PAR_avg(i,j,k) = cff*( PAR_avg(i,j,k) + PAR(i,j,k) )
               end do
            end do
          end do
#  endif /* BIOLOGY_NPZDOC || BIOLOGY_BEC */
# endif /* SOLVE3D */
        endif
      endif
      return
      end
#else
      subroutine set_bio_avg_empty
      end
#endif /* AVERAGES && OUTPUT_AVG_PHYS_BIO */

