#include "cppdefs.h"
                                            ! Setting up curvilinear
      subroutine setup_grid1 (tile)         ! grid: Compute various
      implicit none                         ! combinations of metric
      integer tile                          ! terms.
#include "param.h"
#include "compute_tile_bounds.h"
      call setup_Corls_tile (istr,iend,jstr,jend)
      call setup_grid1_tile (istr,iend,jstr,jend)
      return
      end

      subroutine setup_Corls_tile (istr,iend,jstr,jend)
      implicit none
      integer istr,iend,jstr,jend, i,j
#include "param.h"
#include "scalars.h"
#include "grid.h"
#ifdef SPHERICAL
      real cff1, cff2
#endif

#include "compute_extended_bounds.h"

! Set f/mn,at horizontal RHO-points. Note that Coriolis arrays "f",
! "fomn" are computed over extended bounds MUST NOT be communicated/
! exchanged ! code because of potential conflict with periodicity (if
! selected); same is true for the coordinate variables).

#ifdef SPHERICAL
c**   write(*,*) 'Computing Coriolis...'
      cff1=4.*pi/day2sec
      do j=jstrR,jendR
        do i=istrR,iendR
          f(i,j)=cff1*sin(deg2rad*latr(i,j))
          fomn(i,j)=f(i,j)/(pm(i,j)*pn(i,j))
# ifdef NON_TRADITIONAL
          cff2=cff1*cos(deg2rad*latr(i,j))
          f_XI(i,j)= cff2*sin(angler(i,j))
          f_ETA(i,j)=cff2*cos(angler(i,j))
# endif
        enddo
      enddo
#else
      do j=jstrR,jendR
        do i=istrR,iendR
          fomn(i,j)=f(i,j)/(pm(i,j)*pn(i,j))
        enddo
      enddo
#endif
      return
      end


      subroutine setup_grid1_tile (istr,iend,jstr,jend)
      implicit none
      integer istr,iend,jstr,jend, i,j
#include "param.h"
#include "scalars.h"
#include "grid.h"
#ifdef MASKING
      real cff1, cff2
#endif

#include "compute_auxiliary_bounds.h"

!  Compute n/m and m/n; all at horizontal RHO-points.

      do j=jstrR,jendR
        do i=istrR,iendR
          dm_r(i,j)=1./pm(i,j)
          dn_r(i,j)=1./pn(i,j)
        enddo
      enddo

#if (defined CURVGRID && defined UV_ADV)

!  Compute d(1/n)/d(xi) and d(1/m)/d(eta) tems, both at RHO-points.

      do j=jstrR,jendR
        do i=istr,iend
          dndx(i,j)=0.5/pn(i+1,j)-0.5/pn(i-1,j)
        enddo
      enddo
      do j=jstr,jend
        do i=istrR,iendR
          dmde(i,j)=0.5/pm(i,j+1)-0.5/pm(i,j-1)
        enddo
      enddo

#endif

!  Compute m/n at horizontal U-points.

      do j=jstrR,jendR
        do i=istr,iendR
           pmon_u(i,j)=(pm(i,j)+pm(i-1,j))/(pn(i,j)+pn(i-1,j))
           dm_u(i,j)=2./(pm(i,j)+pm(i-1,j))
           dn_u(i,j)=2./(pn(i,j)+pn(i-1,j))
#ifdef MASKING
           umask(i,j)=rmask(i,j)*rmask(i-1,j)
#endif
           iA_u(i,j)=0.25*(pm(i,j)+pm(i-1,j))*(pn(i,j)+pn(i-1,j))
        enddo
      enddo

!  Compute n/m at horizontal V-points.

      do j=jstr,jendR
        do i=istrR,iendR
          pnom_v(i,j)=(pn(i,j)+pn(i,j-1))/(pm(i,j)+pm(i,j-1))
          dm_v(i,j)=2./(pm(i,j)+pm(i,j-1))
          dn_v(i,j)=2./(pn(i,j)+pn(i,j-1))
#ifdef MASKING
          vmask(i,j)=rmask(i,j)*rmask(i,j-1)
#endif
          iA_v(i,j)=0.25*(pm(i,j)+pm(i,j-1))*(pn(i,j)+pn(i,j-1))
        enddo
      enddo

! Compute n/m and m/n at horizontal PSI-points.
! Set mask according to slipperness parameter gamma.

      do j=jstr,jendR
        do i=istr,iendR
          dm_p(i,j)=4./(pm(i,j)+pm(i,j-1)+pm(i-1,j)+pm(i-1,j-1))
          dn_p(i,j)=4./(pn(i,j)+pn(i,j-1)+pn(i-1,j)+pn(i-1,j-1))

#ifdef MASKING
c          pmask(i,j)=rmask(i,j)*rmask(i-1,j)*rmask(i,j-1)
c     &                                      *rmask(i-1,j-1)
C***      if (gamma2.lt.0.) pmask(i,j)=2.-pmask(i,j)

! Set no-slip boundary conditions on land-mask boundaries
! regardless of supplied value of gamma2.


          cff1=1.  !<-- computation of off-diagonal nonlinear terms
          cff2=2.

          if (rmask(i-1,j  ).gt.0.5 .and. rmask(i,j  ).gt.0.5 .and.
     &        rmask(i-1,j-1).gt.0.5 .and. rmask(i,j-1).gt.0.5) then
            pmask(i,j)=1.

          elseif(rmask(i-1,j  ).lt.0.5 .and.rmask(i,j  ).gt.0.5 .and.
     &           rmask(i-1,j-1).gt.0.5 .and.rmask(i,j-1).gt.0.5) then
            pmask(i,j)=cff1
          elseif(rmask(i-1,j  ).gt.0.5 .and.rmask(i,j  ).lt.0.5 .and.
     &           rmask(i-1,j-1).gt.0.5 .and.rmask(i,j-1).gt.0.5) then
            pmask(i,j)=cff1
          elseif(rmask(i-1,j  ).gt.0.5 .and.rmask(i,j  ).gt.0.5 .and.
     &           rmask(i-1,j-1).lt.0.5 .and.rmask(i,j-1).gt.0.5) then
            pmask(i,j)=cff1
          elseif(rmask(i-1,j  ).gt.0.5 .and.rmask(i,j  ).gt.0.5 .and.
     &           rmask(i-1,j-1).gt.0.5 .and.rmask(i,j-1).lt.0.5) then
            pmask(i,j)=cff1


          elseif(rmask(i-1,j  ).gt.0.5 .and.rmask(i,j  ).lt.0.5 .and.
     &           rmask(i-1,j-1).gt.0.5 .and.rmask(i,j-1).lt.0.5) then
            pmask(i,j)=cff2
          elseif(rmask(i-1,j  ).lt.0.5 .and.rmask(i,j  ).gt.0.5 .and.
     &           rmask(i-1,j-1).lt.0.5 .and.rmask(i,j-1).gt.0.5) then
            pmask(i,j)=cff2
          elseif(rmask(i-1,j  ).gt.0.5 .and.rmask(i,j  ).gt.0.5 .and.
     &           rmask(i-1,j-1).lt.0.5 .and.rmask(i,j-1).lt.0.5) then
            pmask(i,j)=cff2
          elseif(rmask(i-1,j  ).lt.0.5 .and.rmask(i,j  ).lt.0.5 .and.
     &           rmask(i-1,j-1).gt.0.5 .and.rmask(i,j-1).gt.0.5) then
            pmask(i,j)=cff2

          else
            pmask(i,j)=0.
          endif
#endif
        enddo
      enddo

#ifdef EXCHANGE
      call exchange2d_4_tile (istr,iend,jstr,jend,  dm_r, dn_r,
     &                                              dm_p,  dn_p)

      call exchange2d_3_tile(istr,iend,jstr,jend, dm_u, dn_u, iA_u)

      call exchange2d_3_tile(istr,iend,jstr,jend, dm_v, dn_v, iA_v)

# if defined CURVGRID && defined UV_ADV
      call exchange2d_4_tile (istr,iend,jstr,jend, pmon_u, pnom_v,
     &                                               dndx,   dmde)
# else
      call exchange2d_2_tile (istr,iend,jstr,jend, pmon_u, pnom_v)
# endif

# ifdef MASKING
      call exchange2d_4_tile (istr,iend,jstr,jend,  rmask, umask,
     &                                              vmask, pmask)
# endif
#endif /* EXCHANGE */

      return
      end


